@@ -1,6 +1,6 @@
 # -*- test-case-name: twisted.test.test_amp -*-
 # Copyright (c) 2005 Divmod, Inc.
-# Copyright (c) 2007 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -59,12 +59,25 @@ responder for our 'Sum' command::
         Sum.responder(sum)
 
 Later, when you want to actually do a sum, the following expression will return
-a Deferred which will fire with the result::
+a L{Deferred} which will fire with the result::
 
     ClientCreator(reactor, amp.AMP).connectTCP(...).addCallback(
         lambda p: p.callRemote(Sum, a=13, b=81)).addCallback(
             lambda result: result['total'])
 
+Command responders may also return Deferreds, causing the response to be
+sent only once the Deferred fires::
+
+    class DelayedSum(amp.AMP):
+        def slowSum(self, a, b):
+            total = a + b
+            result = defer.Deferred()
+            reactor.callLater(3, result.callback, {'total': total})
+            return result
+        Sum.responder(slowSum)
+
+This is transparent to the caller.
+
 You can also define the propagation of specific errors in AMP.  For example,
 for the slightly more complicated case of division, we might have to deal with
 division by zero::
@@ -129,6 +142,11 @@ so::
 
 Notes:
 
+In general, the order of keys is arbitrary.  Specific uses of AMP may impose an
+ordering requirement, but unless this is specified explicitly, any ordering may
+be generated and any ordering must be accepted.  This applies to the
+command-related keys I{_command} and I{_ask} as well as any other keys.
+
 Values are limited to the maximum encodable size in a 16-bit length, 65535
 bytes.
 
@@ -137,8 +155,8 @@ Note that we still use 2-byte lengths to encode keys.  This small redundancy
 has several features:
 
     - If an implementation becomes confused and starts emitting corrupt data,
-      or gets keys confused with values, many common errors will be
-      signalled immediately instead of delivering obviously corrupt packets.
+      or gets keys confused with values, many common errors will be signalled
+      immediately instead of delivering obviously corrupt packets.
 
     - A single NUL will separate every key, and a double NUL separates
       messages.  This provides some redundancy when debugging traffic dumps.
@@ -151,7 +169,6 @@ has several features:
       plain-text protocol, and easily distinguish between non-AMP clients (like
       web browsers) which issue non-NUL as the first byte, and AMP clients,
       which always issue NUL as the first byte.
-
 """
 
 __metaclass__ = type
@@ -160,24 +177,32 @@ import types, warnings
 
 from cStringIO import StringIO
 from struct import pack
+import decimal, datetime
 
 from zope.interface import Interface, implements
 
+from twisted.python.compat import set
+from twisted.python.util import unsignedID
 from twisted.python.reflect import accumulateClassDict
 from twisted.python.failure import Failure
 from twisted.python import log, filepath
 
 from twisted.internet.main import CONNECTION_LOST
 from twisted.internet.error import PeerVerifyError, ConnectionLost
+from twisted.internet.error import ConnectionClosed
 from twisted.internet.defer import Deferred, maybeDeferred, fail
 from twisted.protocols.basic import Int16StringReceiver, StatefulStringProtocol
 
-from twisted.internet._sslverify import problemsFromTransport
+try:
+    from twisted.internet import ssl
+except ImportError:
+    ssl = None
 
-# I'd like this to use the exposed public API, but for some reason, when it was
-# moved, these names were not exposed by internet.ssl.
+if ssl and not ssl.supported:
+    ssl = None
 
-from twisted.internet.ssl import CertificateOptions, Certificate, DN, KeyPair
+if ssl is not None:
+    from twisted.internet.ssl import CertificateOptions, Certificate, DN, KeyPair
 
 ASK = '_ask'
 ANSWER = '_answer'
@@ -192,6 +217,71 @@ MAX_KEY_LENGTH = 0xff
 MAX_VALUE_LENGTH = 0xffff
 
 
+class IArgumentType(Interface):
+    """
+    An L{IArgumentType} can serialize a Python object into an AMP box and
+    deserialize information from an AMP box back into a Python object.
+
+    @since: 9.0
+    """
+    def fromBox(name, strings, objects, proto):
+        """
+        Given an argument name and an AMP box containing serialized values,
+        extract one or more Python objects and add them to the C{objects}
+        dictionary.
+
+        @param name: The name associated with this argument.  Most commonly,
+            this is the key which can be used to find a serialized value in
+            C{strings} and which should be used as the key in C{objects} to
+            associate with a structured Python object.
+        @type name: C{str}
+
+        @param strings: The AMP box from which to extract one or more
+            values.
+        @type strings: C{dict}
+
+        @param objects: The output dictionary to populate with the value for
+            this argument.
+        @type objects: C{dict}
+
+        @param proto: The protocol instance which received the AMP box being
+            interpreted.  Most likely this is an instance of L{AMP}, but
+            this is not guaranteed.
+
+        @return: C{None}
+        """
+
+
+    def toBox(name, strings, objects, proto):
+        """
+        Given an argument name and a dictionary containing structured Python
+        objects, serialize values into one or more strings and add them to
+        the C{strings} dictionary.
+
+        @param name: The name associated with this argument.  Most commonly,
+            this is the key which can be used to find an object in
+            C{objects} and which should be used as the key in C{strings} to
+            associate with a C{str} giving the serialized form of that
+            object.
+        @type name: C{str}
+
+        @param strings: The AMP box into which to insert one or more
+            strings.
+        @type strings: C{dict}
+
+        @param objects: The input dictionary from which to extract Python
+            objects to serialize.
+        @type objects: C{dict}
+
+        @param proto: The protocol instance which will send the AMP box once
+            it is fully populated.  Most likely this is an instance of
+            L{AMP}, but this is not guaranteed.
+
+        @return: C{None}
+        """
+
+
+
 class IBoxSender(Interface):
     """
     A transport which can send L{AmpBox} objects.
@@ -679,7 +769,7 @@ class BoxDispatcher:
         this command, or None, as specified by requiresAnswer.
         """
         box = Box(kw)
-        return self._sendBoxCommand(command, box)
+        return self._sendBoxCommand(command, box, requiresAnswer)
 
 
     def callRemote(self, commandType, *a, **kw):
@@ -719,7 +809,10 @@ class BoxDispatcher:
         # In other words, the fact that commandType is instantiated here is an
         # implementation detail.  Don't rely on it.
 
-        co = commandType(*a, **kw)
+        try:
+            co = commandType(*a, **kw)
+        except:
+            return fail()
         return co._doCommand(self)
 
 
@@ -767,7 +860,6 @@ class BoxDispatcher:
         @param box: an L{AmpBox} with a value for its L{COMMAND} and L{ASK}
         keys.
         """
-        cmd = box[COMMAND]
         def formatAnswer(answerBox):
             answerBox[ANSWER] = box[ASK]
             return answerBox
@@ -866,6 +958,9 @@ class CommandLocator:
         Command.responder decorator outside the context of a CommandLocator
         class declaration.
 
+        Command responders defined on subclasses are given precedence over
+        those inherited from a base class.
+
         The Command.responder decorator explicitly cooperates with this
         metaclass.
         """
@@ -875,11 +970,13 @@ class CommandLocator:
             commands = cls._currentClassCommands[:]
             cls._currentClassCommands[:] = []
             cd = attrs['_commandDispatch'] = {}
-            for base in bases:
-                cls._grabFromBase(cd, base)
+            subcls = type.__new__(cls, name, bases, attrs)
+            ancestors = list(subcls.__mro__[1:])
+            ancestors.reverse()
+            for ancestor in ancestors:
+                cd.update(getattr(ancestor, '_commandDispatch', {}))
             for commandClass, responderFunc in commands:
                 cd[commandClass.commandName] = (commandClass, responderFunc)
-            subcls = type.__new__(cls, name, bases, attrs)
             if (bases and (
                     subcls.lookupFunction != CommandLocator.lookupFunction)):
                 def locateResponder(self, name):
@@ -891,12 +988,6 @@ class CommandLocator:
                 subcls.locateResponder = locateResponder
             return subcls
 
-        def _grabFromBase(cls, cd, base):
-            if hasattr(base, "_commandDispatch"):
-                cd.update(base._commandDispatch)
-                for subbase in base.__bases__:
-                    cls._grabFromBase(cd, subbase)
-        _grabFromBase = classmethod(_grabFromBase)
 
     implements(IResponderLocator)
 
@@ -1043,7 +1134,14 @@ class Argument:
     """
     Base-class of all objects that take values from Amp packets and convert
     them into objects for Python functions.
+
+    This implementation of L{IArgumentType} provides several higher-level
+    hooks for subclasses to override.  See L{toString} and L{fromString}
+    which will be used to define the behavior of L{IArgumentType.toBox} and
+    L{IArgumentType.fromBox}, respectively.
     """
+    implements(IArgumentType)
+
     optional = False
 
 
@@ -1186,7 +1284,10 @@ class Argument:
 
 class Integer(Argument):
     """
-    Convert to and from 'int'.
+    Encode any integer values of any size on the wire as the string
+    representation.
+
+    Example: C{123} becomes C{"123"}
     """
     fromString = int
     def toString(self, inObject):
@@ -1271,6 +1372,63 @@ class Path(Unicode):
 
 
 
+class ListOf(Argument):
+    """
+    Encode and decode lists of instances of a single other argument type.
+
+    For example, if you want to pass::
+
+        [3, 7, 9, 15]
+
+    You can create an argument like this::
+
+        ListOf(Integer())
+
+    The serialized form of the entire list is subject to the limit imposed by
+    L{MAX_VALUE_LENGTH}.  List elements are represented as 16-bit length
+    prefixed strings.  The argument type passed to the L{ListOf} initializer is
+    responsible for producing the serialized form of each element.
+
+    @ivar elementType: The L{Argument} instance used to encode and decode list
+        elements (note, not an arbitrary L{IArgument} implementation:
+        arguments must be implemented using only the C{fromString} and
+        C{toString} methods, not the C{fromBox} and C{toBox} methods).
+
+    @param optional: a boolean indicating whether this argument can be
+        omitted in the protocol.
+
+    @since: 10.0
+    """
+    def __init__(self, elementType, optional=False):
+        self.elementType = elementType
+        Argument.__init__(self, optional)
+
+
+    def fromString(self, inString):
+        """
+        Convert the serialized form of a list of instances of some type back
+        into that list.
+        """
+        strings = []
+        parser = Int16StringReceiver()
+        parser.stringReceived = strings.append
+        parser.dataReceived(inString)
+        return map(self.elementType.fromString, strings)
+
+
+    def toString(self, inObject):
+        """
+        Serialize the given list of objects to a single string.
+        """
+        strings = []
+        for obj in inObject:
+            serialized = self.elementType.toString(obj)
+            strings.append(pack('!H', len(serialized)))
+            strings.append(serialized)
+        return ''.join(strings)
+
+
+
 class AmpList(Argument):
     """
     Convert a list of dictionaries into a list of AMP boxes on the wire.
@@ -1284,14 +1442,18 @@ class AmpList(Argument):
         AmpList([('a', Integer()),
                  ('b', Unicode())])
     """
-    def __init__(self, subargs):
+    def __init__(self, subargs, optional=False):
         """
         Create an AmpList.
 
         @param subargs: a list of 2-tuples of ('name', argument) describing the
         schema of the dictionaries in the sequence of amp boxes.
+
+        @param optional: a boolean indicating whether this argument can be
+        omitted in the protocol.
         """
         self.subargs = subargs
+        Argument.__init__(self, optional)
 
 
     def fromStringProto(self, inString, proto):
@@ -1353,7 +1515,7 @@ class Command:
         'fatalErrors' as class vars.
         """
         def __new__(cls, name, bases, attrs):
-            re = attrs['reverseErrors'] = {}
+            reverseErrors = attrs['reverseErrors'] = {}
             er = attrs['allErrors'] = {}
             if 'commandName' not in attrs:
                 attrs['commandName'] = name
@@ -1363,10 +1525,10 @@ class Command:
             accumulateClassDict(newtype, 'errors', errors)
             accumulateClassDict(newtype, 'fatalErrors', fatalErrors)
             for v, k in errors.iteritems():
-                re[k] = v
+                reverseErrors[k] = v
                 er[v] = k
             for v, k in fatalErrors.iteritems():
-                re[k] = v
+                reverseErrors[k] = v
                 er[v] = k
             return newtype
 
@@ -1418,8 +1580,11 @@ class Command:
 
         @return: an L{AmpBox}.
         """
-        return _objectsToStrings(objects, cls.response, cls.responseType(),
-                                 proto)
+        try:
+            responseType = cls.responseType()
+        except:
+            return fail()
+        return _objectsToStrings(objects, cls.response, responseType, proto)
     makeResponse = classmethod(makeResponse)
 
 
@@ -1436,6 +1601,14 @@ class Command:
 
         @return: An instance of this L{Command}'s C{commandType}.
         """
+        allowedNames = set()
+        for (argName, ignored) in cls.arguments:
+            allowedNames.add(_wireNameToPythonIdentifier(argName))
+
+        for intendedArg in objects:
+            if intendedArg not in allowedNames:
+                raise InvalidSignature(
+                    "%s is not a valid argument" % (intendedArg,))
         return _objectsToStrings(objects, cls.arguments, cls.commandType(),
                                  proto)
     makeArguments = classmethod(makeArguments)
@@ -1599,6 +1772,12 @@ class _TLSBox(AmpBox):
     """
     __slots__ = []
 
+    def __init__(self):
+        if ssl is None:
+            raise RemoteAmpError("TLS_ERROR", "TLS not available")
+        AmpBox.__init__(self)
+
+
     def _keyprop(k, default):
         return property(lambda self: self.get(k, default))
 
@@ -1670,6 +1849,8 @@ class StartTLS(Command):
         @param tls_verifyAuthorities: a list of Certificate objects which
         represent root certificates to verify our peer with.
         """
+        if ssl is None:
+            raise RuntimeError("TLS not available.")
         self.certificate = kw.pop('tls_localCertificate', _NoCertificate(True))
         self.authorities = kw.pop('tls_verifyAuthorities', None)
         Command.__init__(self, **kw)
@@ -1767,6 +1948,10 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
     tricky elements of the implementation, the public interface to this
     functionality is L{ProtocolSwitchCommand} and L{StartTLS}.
 
+    @ivar _keyLengthLimitExceeded: A flag which is only true when the
+        connection is being closed because a key length prefix which was longer
+        than allowed by the protocol was received.
+
     @ivar boxReceiver: an L{IBoxReceiver} provider, whose L{ampBoxReceived}
     method will be invoked for each L{Box} that is received.
     """
@@ -1779,19 +1964,13 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
     _currentKey = None
     _currentBox = None
 
+    _keyLengthLimitExceeded = False
+
     hostCertificate = None
     noPeerCertificate = False   # for tests
     innerProtocol = None
     innerProtocolClientFactory = None
 
-    _sslVerifyProblems = ()
-    # ^ Later this will become a mutable list - we can't get the handle during
-    # connection shutdown thanks to the fact that Twisted destroys the socket
-    # on our transport before notifying us of a lost connection (which I guess
-    # is reasonable - the socket is dead by then) See a few lines below in
-    # startTLS for details.  --glyph
-
-
     def __init__(self, boxReceiver):
         self.boxReceiver = boxReceiver
 
@@ -1821,7 +2000,8 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         self.innerProtocol = newProto
         self.innerProtocolClientFactory = clientFactory
         newProto.makeConnection(self.transport)
-        newProto.dataReceived(newProtoData)
+        if newProtoData:
+            newProto.dataReceived(newProtoData)
 
 
     def sendBox(self, box):
@@ -1852,8 +2032,9 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         Notify L{boxReceiver} that it is about to receive boxes from this
         protocol by invoking L{startReceivingBoxes}.
         """
+        self.transport = transport
         self.boxReceiver.startReceivingBoxes(self)
-        Int16StringReceiver.makeConnection(self, transport)
+        self.connectionMade()
 
 
     def dataReceived(self, data):
@@ -1879,15 +2060,9 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
             self.innerProtocol.connectionLost(reason)
             if self.innerProtocolClientFactory is not None:
                 self.innerProtocolClientFactory.clientConnectionLost(None, reason)
-        # XXX this may be a slight oversimplification, but I believe that if
-        # there are pending SSL errors, they _are_ the reason that the
-        # connection was lost.  a totally correct implementation of this would
-        # set up a simple state machine to track whether any bytes were
-        # received after startTLS was called.  --glyph
-        problems = self._sslVerifyProblems
-        if problems:
-            failReason = Failure(problems[0])
-        elif self._justStartedTLS:
+        if self._keyLengthLimitExceeded:
+            failReason = Failure(TooLong(True, False, None, None))
+        elif reason.check(ConnectionClosed) and self._justStartedTLS:
             # We just started TLS and haven't received any data.  This means
             # the other connection didn't like our cert (although they may not
             # have told us why - later Twisted should make 'reason' into a TLS
@@ -1899,6 +2074,15 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         self.boxReceiver.stopReceivingBoxes(failReason)
 
 
+    # The longest key allowed
+    _MAX_KEY_LENGTH = 255
+
+    # The longest value allowed (this is somewhat redundant, as longer values
+    # cannot be encoded - ah well).
+    _MAX_VALUE_LENGTH = 65535
+
+    # The first thing received is a key.
+    MAX_LENGTH = _MAX_KEY_LENGTH
 
     def proto_init(self, string):
         """
@@ -1915,6 +2099,7 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         """
         if string:
             self._currentKey = string
+            self.MAX_LENGTH = self._MAX_VALUE_LENGTH
             return 'value'
         else:
             self.boxReceiver.ampBoxReceived(self._currentBox)
@@ -1928,9 +2113,19 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         """
         self._currentBox[self._currentKey] = string
         self._currentKey = None
+        self.MAX_LENGTH = self._MAX_KEY_LENGTH
         return 'key'
 
 
+    def lengthLimitExceeded(self, length):
+        """
+        The key length limit was exceeded.  Disconnect the transport and make
+        sure a meaningful exception is reported.
+        """
+        self._keyLengthLimitExceeded = True
+        self.transport.loseConnection()
+
+
     def _lockForSwitch(self):
         """
         Lock this binary protocol so that no further boxes may be sent.  This
@@ -1983,10 +2178,6 @@ class BinaryBoxProtocol(StatefulStringProtocol, Int16StringReceiver):
         if verifyAuthorities is None:
             verifyAuthorities = ()
         self.transport.startTLS(certificate.options(*verifyAuthorities))
-        # Remember that mutable list that we were just talking about?  Here
-        # it is.  sslverify.py takes care of populating this list as
-        # necessary. --glyph
-        self._sslVerifyProblems = problemsFromTransport(self.transport)
         stlsb = self._startingTLSBuffer
         if stlsb is not None:
             self._startingTLSBuffer = None
@@ -2052,7 +2243,6 @@ class AMP(BinaryBoxProtocol, BoxDispatcher,
             boxReceiver = self
         if locator is None:
             locator = self
-        boxSender = self
         BoxDispatcher.__init__(self, locator)
         BinaryBoxProtocol.__init__(self, boxReceiver)
 
@@ -2075,9 +2265,12 @@ class AMP(BinaryBoxProtocol, BoxDispatcher,
         A verbose string representation which gives us information about this
         AMP connection.
         """
-        return '<%s %s at 0x%x>' % (
-            self.__class__.__name__,
-            self.innerProtocol, id(self))
+        if self.innerProtocol is not None:
+            innerRepr = ' inner %r' % (self.innerProtocol,)
+        else:
+            innerRepr = ''
+        return '<%s%s at 0x%x>' % (
+            self.__class__.__name__, innerRepr, unsignedID(self))
 
 
     def makeConnection(self, transport):
@@ -2221,3 +2414,168 @@ def _objectsToStrings(objects, arglist, strings, proto):
     return strings
 
 
+
+class _FixedOffsetTZInfo(datetime.tzinfo):
+    """
+    Represents a fixed timezone offset (without daylight saving time).
+
+    @ivar name: A C{str} giving the name of this timezone; the name just
+        includes how much time this offset represents.
+
+    @ivar offset: A C{datetime.timedelta} giving the amount of time this
+        timezone is offset.
+    """
+
+    def __init__(self, sign, hours, minutes):
+        self.name = '%s%02i:%02i' % (sign, hours, minutes)
+        if sign == '-':
+            hours = -hours
+            minutes = -minutes
+        elif sign != '+':
+            raise ValueError('invalid sign for timezone %r' % (sign,))
+        self.offset = datetime.timedelta(hours=hours, minutes=minutes)
+
+
+    def utcoffset(self, dt):
+        """
+        Return this timezone's offset from UTC.
+        """
+        return self.offset
+
+
+    def dst(self, dt):
+        """
+        Return a zero C{datetime.timedelta} for the daylight saving time offset,
+        since there is never one.
+        """
+        return datetime.timedelta(0)
+
+
+    def tzname(self, dt):
+        """
+        Return a string describing this timezone.
+        """
+        return self.name
+
+
+
+utc = _FixedOffsetTZInfo('+', 0, 0)
+
+
+
+class Decimal(Argument):
+    """
+    Encodes C{decimal.Decimal} instances.
+
+    There are several ways in which a decimal value might be encoded.
+
+    Special values are encoded as special strings::
+
+      - Positive infinity is encoded as C{"Infinity"}
+      - Negative infinity is encoded as C{"-Infinity"}
+      - Quiet not-a-number is encoded as either C{"NaN"} or C{"-NaN"}
+      - Signalling not-a-number is encoded as either C{"sNaN"} or C{"-sNaN"}
+
+    Normal values are encoded using the base ten string representation, using
+    engineering notation to indicate magnitude without precision, and "normal"
+    digits to indicate precision.  For example::
+
+      - C{"1"} represents the value I{1} with precision to one place.
+      - C{"-1"} represents the value I{-1} with precision to one place.
+      - C{"1.0"} represents the value I{1} with precision to two places.
+      - C{"10"} represents the value I{10} with precision to two places.
+      - C{"1E+2"} represents the value I{10} with precision to one place.
+      - C{"1E-1"} represents the value I{0.1} with precision to one place.
+      - C{"1.5E+2"} represents the value I{15} with precision to two places.
+
+    U{http://speleotrove.com/decimal/} should be considered the authoritative
+    specification for the format.
+    """
+    fromString = decimal.Decimal
+
+    def toString(self, inObject):
+        """
+        Serialize a C{decimal.Decimal} instance to the specified wire format.
+        """
+        if isinstance(inObject, decimal.Decimal):
+            # Hopefully decimal.Decimal.__str__ actually does what we want.
+            return str(inObject)
+        raise ValueError(
+            "amp.Decimal can only encode instances of decimal.Decimal")
+
+
+
+class DateTime(Argument):
+    """
+    Encodes C{datetime.datetime} instances.
+
+    Wire format: '%04i-%02i-%02iT%02i:%02i:%02i.%06i%s%02i:%02i'. Fields in
+    order are: year, month, day, hour, minute, second, microsecond, timezone
+    direction (+ or -), timezone hour, timezone minute. Encoded string is
+    always exactly 32 characters long. This format is compatible with ISO 8601,
+    but that does not mean all ISO 8601 dates can be accepted.
+
+    Also, note that the datetime module's notion of a "timezone" can be
+    complex, but the wire format includes only a fixed offset, so the
+    conversion is not lossless. A lossless transmission of a C{datetime} instance
+    is not feasible since the receiving end would require a Python interpreter.
+
+    @ivar _positions: A sequence of slices giving the positions of various
+        interesting parts of the wire format.
+    """
+
+    _positions = [
+        slice(0, 4), slice(5, 7), slice(8, 10), # year, month, day
+        slice(11, 13), slice(14, 16), slice(17, 19), # hour, minute, second
+        slice(20, 26), # microsecond
+        # intentionally skip timezone direction, as it is not an integer
+        slice(27, 29), slice(30, 32) # timezone hour, timezone minute
+        ]
+
+    def fromString(self, s):
+        """
+        Parse a string containing a date and time in the wire format into a
+        C{datetime.datetime} instance.
+        """
+        if len(s) != 32:
+            raise ValueError('invalid date format %r' % (s,))
+
+        values = [int(s[p]) for p in self._positions]
+        sign = s[26]
+        timezone = _FixedOffsetTZInfo(sign, *values[7:])
+        values[7:] = [timezone]
+        return datetime.datetime(*values)
+
+
+    def toString(self, i):
+        """
+        Serialize a C{datetime.datetime} instance to a string in the specified
+        wire format.
+        """
+        offset = i.utcoffset()
+        if offset is None:
+            raise ValueError(
+                'amp.DateTime cannot serialize naive datetime instances.  '
+                'You may find amp.utc useful.')
+
+        minutesOffset = (offset.days * 86400 + offset.seconds) // 60
+
+        if minutesOffset > 0:
+            sign = '+'
+        else:
+            sign = '-'
+
+        # strftime has no way to format the microseconds, or put a ':' in the
+        # timezone. Suprise!
+
+        return '%04i-%02i-%02iT%02i:%02i:%02i.%06i%s%02i:%02i' % (
+            i.year,
+            i.month,
+            i.day,
+            i.hour,
+            i.minute,
+            i.second,
+            i.microsecond,
+            sign,
+            abs(minutesOffset) // 60,
+            abs(minutesOffset) % 60)
