@@ -1,7 +1,7 @@
 # -*- test-case-name: twisted.test.test_failure -*-
 # See also test suite twisted.test.test_pbfailure
 
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -119,7 +119,8 @@ class _Code(object):
 
 
 class Failure:
-    """A basic abstraction for an error that has occurred.
+    """
+    A basic abstraction for an error that has occurred.
 
     This is necessary because Python's built-in error mechanisms are
     inconvenient for asynchronous communication.
@@ -137,18 +138,24 @@ class Failure:
     _yieldOpcode = chr(opcode.opmap["YIELD_VALUE"])
 
     def __init__(self, exc_value=None, exc_type=None, exc_tb=None):
-        """Initialize me with an explanation of the error.
+        """
+        Initialize me with an explanation of the error.
 
-        By default, this will use the current X{exception}
+        By default, this will use the current C{exception}
         (L{sys.exc_info}()).  However, if you want to specify a
         particular kind of failure, you can pass an exception as an
         argument.
 
-        If no C{exc_value} is passed, then an "original" Failure will
+        If no C{exc_value} is passed, then an "original" C{Failure} will
         be searched for. If the current exception handler that this
-        Failure is being constructed in is handling an exception
-        raised by L{raiseException}, then this Failure will act like
-        the original Failure.
+        C{Failure} is being constructed in is handling an exception
+        raised by L{raiseException}, then this C{Failure} will act like
+        the original C{Failure}.
+
+        For C{exc_tb} only L{traceback} instances or C{None} are allowed.
+        If C{None} is supplied for C{exc_value}, the value of C{exc_tb} is
+        ignored, otherwise if C{exc_tb} is C{None}, it will be found from
+        execution context (ie, L{sys.exc_info}).
         """
         global count
         count = count + 1
@@ -275,15 +282,15 @@ class Failure:
         automatically re-raised if it is not a type that you expect.
 
         The reason for having this particular API is because it's very useful
-        in Deferred errback chains:
+        in Deferred errback chains::
 
-        | def _ebFoo(self, failure):
-        |     r = failure.trap(Spam, Eggs)
-        |     print 'The Failure is due to either Spam or Eggs!'
-        |     if r == Spam:
-        |         print 'Spam did it!'
-        |     elif r == Eggs:
-        |         print 'Eggs did it!'
+            def _ebFoo(self, failure):
+                r = failure.trap(Spam, Eggs)
+                print 'The Failure is due to either Spam or Eggs!'
+                if r == Spam:
+                    print 'Spam did it!'
+                elif r == Eggs:
+                    print 'Eggs did it!'
 
         If the failure is not a Spam or an Eggs, then the Failure
         will be 'passed on' to the next errback.
@@ -532,10 +539,17 @@ DO_POST_MORTEM = True
 
 def _debuginit(self, exc_value=None, exc_type=None, exc_tb=None,
              Failure__init__=Failure.__init__.im_func):
+    """
+    Initialize failure object, possibly spawning pdb.
+    """
     if (exc_value, exc_type, exc_tb) == (None, None, None):
         exc = sys.exc_info()
         if not exc[0] == self.__class__ and DO_POST_MORTEM:
-            print "Jumping into debugger for post-mortem of exception '%s':" % exc[1]
+            try:
+                strrepr = str(exc[1])
+            except:
+                strrepr = "broken str"
+            print "Jumping into debugger for post-mortem of exception '%s':" % (strrepr,)
             import pdb
             pdb.post_mortem(exc[2])
     Failure__init__(self, exc_value, exc_type, exc_tb)
