@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_modules -*-
-# Copyright (c) 2006-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2006-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -61,6 +61,7 @@ from os.path import dirname, split as splitpath
 import sys
 import zipimport
 import inspect
+import warnings
 from zope.interface import Interface, implements
 
 from twisted.python.components import registerAdapter
@@ -208,7 +209,7 @@ class _ModuleIteratorHelper:
         @param modname: a str naming a module to be loaded.  For entries, this
         is a top-level, undotted package name, and for packages it is the name
         of the module without the package prefix.  For example, if you have a
-        PythonModule representing the 'twisted' package, you could use:
+        PythonModule representing the 'twisted' package, you could use::
 
             twistedPackageObj['python']['modules']
 
@@ -324,13 +325,15 @@ class PythonModule(_ModuleIteratorHelper):
         """
         return 'PythonModule<%r>' % (self.name,)
 
+
     def isLoaded(self):
         """
         Determine if the module is loaded into sys.modules.
 
         @return: a boolean: true if loaded, false if not.
         """
-        return self.name in self.pathEntry.pythonPath.moduleDict
+        return self.pathEntry.pythonPath.moduleDict.get(self.name) is not None
+
 
     def iterAttributes(self):
         """
@@ -526,7 +529,7 @@ class PythonPath:
     I represent the very top of the Python object-space, the module list in
     sys.path and the modules list in sys.modules.
 
-    @ivar sysPath: a sequence of strings like sys.path.  This attribute is
+    @ivar _sysPath: a sequence of strings like sys.path.  This attribute is
     read-only.
 
     @ivar moduleDict: a dictionary mapping string module names to module
@@ -592,35 +595,17 @@ class PythonPath:
         self.moduleDict = moduleDict
         self.sysPathHooks = sysPathHooks
         self.importerCache = importerCache
-        self._moduleLoader = moduleLoader
+        self.moduleLoader = moduleLoader
 
 
     def _getSysPath(self):
         """
-        Retrieve the current value of sys.path.
+        Retrieve the current value of the module search path list.
         """
         return self._sysPathFactory()
 
     sysPath = property(_getSysPath)
 
-    def moduleLoader(self, modname):
-        """
-        Replicate python2.4+ sys.modules preservation behavior.
-
-        @param modname: a str module name.
-
-        @return: an imported module.
-
-        @raise: any type of exception that may arise from importing.
-        """
-        freezeModules = self.moduleDict.copy()
-        try:
-            return self._moduleLoader(modname)
-        except:
-            self.moduleDict.clear()
-            self.moduleDict.update(freezeModules)
-            raise
-
     def _findEntryPathString(self, modobj):
         """
         Determine where a given Python module object came from by looking at path
@@ -639,13 +624,18 @@ class PythonPath:
             # the module is completely top-level, not within any packages.  The
             # path entry it's on is just its dirname.
             rval = dirname(topPackageObj.__file__)
+
         # There are probably some awful tricks that an importer could pull
         # which would break this, so let's just make sure... it's a loaded
         # module after all, which means that its path MUST be in
         # path_importer_cache according to PEP 302 -glyph
-        from pprint import pformat
-        assert rval in self.importerCache, '%r for %r not in import cache %s' % (
-            rval, modobj, pformat(self.importerCache))
+        if rval not in self.importerCache:
+            warnings.warn(
+                "%s (for module %s) not in path importer cache "
+                "(PEP 302 violation - check your local configuration)." % (
+                    rval, modobj.__name__),
+                stacklevel=3)
+
         return rval
 
     def _smartPath(self, pathName):
@@ -662,7 +652,7 @@ class PythonPath:
             for hook in self.sysPathHooks:
                 try:
                     importr = hook(pathName)
-                except ImportError, ie:
+                except ImportError:
                     pass
             if importr is _nothing: # still
                 importr = None
@@ -678,18 +668,26 @@ class PythonPath:
             fp = self._smartPath(pathName)
             yield PathEntry(fp, self)
 
+
     def __getitem__(self, modname):
         """
-        Get a python module by a given fully-qualified name.
+        Get a python module by its given fully-qualified name.
 
-        @return: a PythonModule object.
+        @param modname: The fully-qualified Python module name to load.
 
-        @raise: KeyError, if the module name is a module name.
+        @type modname: C{str}
+
+        @return: an object representing the module identified by C{modname}
+
+        @rtype: L{PythonModule}
+
+        @raise KeyError: if the module name is not a valid module name, or no
+            such module can be identified as loadable.
         """
         # See if the module is already somewhere in Python-land.
-        if modname in self.moduleDict:
+        moduleObject = self.moduleDict.get(modname)
+        if moduleObject is not None:
             # we need 2 paths; one of the path entry and one for the module.
-            moduleObject = self.moduleDict[modname]
             pe = PathEntry(
                 self._smartPath(
                     self._findEntryPathString(moduleObject)),
@@ -710,6 +708,7 @@ class PythonPath:
                 return module
         raise KeyError(modname)
 
+
     def __repr__(self):
         """
         Display my sysPath and moduleDict in a string representation.
