@@ -1,9 +1,7 @@
 # -*- test-case-name: twisted.test.test_jelly -*-
-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 S-expression-based persistence of python objects.
 
@@ -61,11 +59,9 @@ Python 2.3; this can be accomplished by using L{twisted.python.compat.set}.
 
 The same rule applies for C{frozenset} and C{sets.ImmutableSet}.
 
-@author: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
+@author: Glyph Lefkowitz
 """
 
-__version__ = "$Revision: 1.48 $"[11:-2]
-
 # System Imports
 import pickle
 import types
@@ -117,17 +113,11 @@ from twisted.python.reflect import namedObject, qual
 from twisted.persisted.crefutil import NotKnown, _Tuple, _InstanceMethod
 from twisted.persisted.crefutil import _DictKeyAndValue, _Dereference
 from twisted.persisted.crefutil import _Container
-from twisted.python import runtime
+from twisted.python.compat import reduce
 
 from twisted.spread.interfaces import IJellyable, IUnjellyable
 
-
-if runtime.platform.getType() == "java":
-    from org.python.core import PyStringMap
-    DictTypes = (DictionaryType, PyStringMap)
-else:
-    DictTypes = (DictionaryType,)
-
+DictTypes = (DictionaryType,)
 
 None_atom = "None"                  # N
 # code
@@ -157,20 +147,29 @@ unpersistable_atom = "unpersistable"# u
 unjellyableRegistry = {}
 unjellyableFactoryRegistry = {}
 
+_NO_STATE = object()
 
-
-def _newInstance(cls, state):
+def _newInstance(cls, state=_NO_STATE):
     """
-    Make a new instance of a class without calling its __init__ method.
-    'state' will be used to update inst.__dict__ . Supports both new- and
-    old-style classes.
+    Make a new instance of a class without calling its __init__ method. 
+    Supports both new- and old-style classes.
+
+    @param state: A C{dict} used to update C{inst.__dict__} or C{_NO_STATE}
+        to skip this part of initialization.
+
+    @return: A new instance of C{cls}.
     """
     if not isinstance(cls, types.ClassType):
         # new-style
         inst = cls.__new__(cls)
-        inst.__dict__.update(state) # Copy 'instance' behaviour
+
+        if state is not _NO_STATE:
+            inst.__dict__.update(state) # Copy 'instance' behaviour
     else:
-        inst = instance(cls, state)
+        if state is not _NO_STATE:
+            inst = instance(cls, state)
+        else:   
+            inst = instance(cls)
     return inst
 
 
@@ -195,7 +194,7 @@ def setUnjellyableForClass(classname, unjellyable):
     If you have written a Copyable class that you expect your client to be
     receiving, write a local "copy" class to represent it, then call::
 
-        jellier.setUnjellyableForClass('module.package.Class', MyJellier).
+        jellier.setUnjellyableForClass('module.package.Class', MyCopier).
 
     Call this at the module level immediately after its class
     definition. MyCopier should be a subclass of RemoteCopy.
@@ -218,7 +217,7 @@ def setUnjellyableFactoryForClass(classname, copyFactory):
     """
     Set the factory to construct a remote instance of a type::
 
-        jellier.setFactoryForClass('module.package.Class', MyFactory)
+      jellier.setUnjellyableFactoryForClass('module.package.Class', MyFactory)
 
     Call this at the module level immediately after its class definition.
     C{copyFactory} should return an instance or subclass of
@@ -240,10 +239,10 @@ def setUnjellyableForClassTree(module, baseClass, prefix=None):
     Set all classes in a module derived from C{baseClass} as copiers for
     a corresponding remote class.
 
-    When you have a heirarchy of Copyable (or Cacheable) classes on
-    one side, and a mirror structure of Copied (or RemoteCache)
-    classes on the other, use this to setCopierForClass all your
-    Copieds for the Copyables.
+    When you have a heirarchy of Copyable (or Cacheable) classes on one
+    side, and a mirror structure of Copied (or RemoteCache) classes on the
+    other, use this to setUnjellyableForClass all your Copieds for the
+    Copyables.
 
     Each copyTag (the \"classname\" argument to getTypeToCopyFor, and
     what the Copyable's getTypeToCopyFor returns) is formed from
@@ -678,7 +677,7 @@ class _Unjellier:
             if not self.taster.isClassAllowed(clz):
                 raise InsecureJelly("Class %s not allowed." % jelType)
             if hasattr(clz, "__setstate__"):
-                ret = _newInstance(clz, {})
+                ret = _newInstance(clz)
                 state = self.unjelly(obj[1])
                 ret.__setstate__(state)
             else:
@@ -868,7 +867,8 @@ class _Unjellier:
         if not self.taster.isModuleAllowed(modName):
             raise InsecureJelly("module %s not allowed" % modName)
         klaus = namedObject(rest[0])
-        if type(klaus) is not types.ClassType:
+        objType = type(klaus)
+        if objType not in (types.ClassType, types.TypeType):
             raise InsecureJelly(
                 "class %r unjellied to something that isn't a class: %r" % (
                     rest[0], klaus))
