@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -7,15 +7,15 @@ FTP tests.
 Maintainer: Andrew Bennetts
 """
 
-import os.path
-from StringIO import StringIO
+import os
 import errno
+from StringIO import StringIO
 
 from zope.interface import implements
 
 from twisted.trial import unittest, util
 from twisted.protocols import basic
-from twisted.internet import reactor, protocol, defer, error
+from twisted.internet import reactor, task, protocol, defer, error
 from twisted.internet.interfaces import IConsumer
 from twisted.cred import portal, checkers, credentials
 from twisted.python import failure, filepath
@@ -23,12 +23,6 @@ from twisted.test import proto_helpers
 
 from twisted.protocols import ftp, loopback
 
-class NonClosingStringIO(StringIO):
-    def close(self):
-        pass
-
-StringIOWithoutClosing = NonClosingStringIO
-
 
 _changeDirectorySuppression = util.suppress(
     category=DeprecationWarning,
@@ -62,6 +56,7 @@ class _BufferingProtocol(protocol.Protocol):
         self.d.callback(self)
 
 
+
 class FTPServerTestCase(unittest.TestCase):
     """
     Simple tests for an FTP server with the default settings.
@@ -75,6 +70,7 @@ class FTPServerTestCase(unittest.TestCase):
         # Create a directory
         self.directory = self.mktemp()
         os.mkdir(self.directory)
+        self.dirPath = filepath.FilePath(self.directory)
 
         # Start the server
         p = portal.Portal(ftp.FTPRealm(self.directory))
@@ -90,6 +86,9 @@ class FTPServerTestCase(unittest.TestCase):
         buildProtocol = self.factory.buildProtocol
         d1 = defer.Deferred()
         def _rememberProtocolInstance(addr):
+            # Done hooking this.
+            del self.factory.buildProtocol
+
             protocol = buildProtocol(addr)
             self.serverProtocol = protocol.wrappedProtocol
             def cleanupServer():
@@ -368,6 +367,18 @@ class BasicFTPServerTestCase(FTPServerTestCase):
                           protocol.Factory())
 
 
+    def test_portRangeInheritedFromFactory(self):
+        """
+        The L{FTP} instances created by L{ftp.FTPFactory.buildProtocol} have
+        their C{passivePortRange} attribute set to the same object the
+        factory's C{passivePortRange} attribute is set to.
+        """
+        portRange = xrange(2017, 2031)
+        self.factory.passivePortRange = portRange
+        protocol = self.factory.buildProtocol(None)
+        self.assertEqual(portRange, protocol.wrappedProtocol.passivePortRange)
+
+
 
 class FTPServerTestCaseAdvancedClient(FTPServerTestCase):
     """
@@ -485,6 +496,58 @@ class FTPServerPasvDataConnectionTestCase(FTPServerTestCase):
         return d
 
 
+    def test_NLSTEmpty(self):
+        """
+        NLST with no argument returns the directory listing for the current
+        working directory.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        # Touch a file in the current working directory
+        self.dirPath.child('test.txt').touch()
+        # Make a directory in the current working directory
+        self.dirPath.child('foo').createDirectory()
+
+        self._download('NLST ', chainDeferred=d)
+        def checkDownload(download):
+            filenames = download[:-2].split('\r\n')
+            filenames.sort()
+            self.assertEquals(['foo', 'test.txt'], filenames)
+        return d.addCallback(checkDownload)
+
+
+    def test_NLSTNonexistent(self):
+        """
+        NLST on a non-existent file/directory returns nothing.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        self._download('NLST nonexistent.txt', chainDeferred=d)
+        def checkDownload(download):
+            self.assertEquals('', download)
+        return d.addCallback(checkDownload)
+
+
+    def test_NLSTOnPathToFile(self):
+        """
+        NLST on an existent file returns only the path to that file.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        # Touch a file in the current working directory
+        self.dirPath.child('test.txt').touch()
+
+        self._download('NLST test.txt', chainDeferred=d)
+        def checkDownload(download):
+            filenames = download[:-2].split('\r\n')
+            self.assertEquals(['test.txt'], filenames)
+        return d.addCallback(checkDownload)
+
+
+
 class FTPServerPortDataConnectionTestCase(FTPServerPasvDataConnectionTestCase):
     def setUp(self):
         self.dataPorts = []
@@ -537,6 +600,143 @@ class FTPServerPortDataConnectionTestCase(FTPServerPasvDataConnectionTestCase):
         return d.addCallback(gotPortNum)
 
 
+
+class DTPFactoryTests(unittest.TestCase):
+    """
+    Tests for L{ftp.DTPFactory}.
+    """
+    def setUp(self):
+        """
+        Create a fake protocol interpreter and a L{ftp.DTPFactory} instance to
+        test.
+        """
+        self.reactor = task.Clock()
+
+        class ProtocolInterpreter(object):
+            dtpInstance = None
+
+        self.protocolInterpreter = ProtocolInterpreter()
+        self.factory = ftp.DTPFactory(
+            self.protocolInterpreter, None, self.reactor)
+
+
+    def test_setTimeout(self):
+        """
+        L{ftp.DTPFactory.setTimeout} uses the reactor passed to its initializer
+        to set up a timed event to time out the DTP setup after the specified
+        number of seconds.
+        """
+        # Make sure the factory's deferred fails with the right exception, and
+        # make it so we can tell exactly when it fires.
+        finished = []
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+        d.addCallback(finished.append)
+
+        self.factory.setTimeout(6)
+
+        # Advance the clock almost to the timeout
+        self.reactor.advance(5)
+
+        # Nothing should have happened yet.
+        self.assertFalse(finished)
+
+        # Advance it to the configured timeout.
+        self.reactor.advance(1)
+
+        # Now the Deferred should have failed with TimeoutError.
+        self.assertTrue(finished)
+
+        # There should also be no calls left in the reactor.
+        self.assertFalse(self.reactor.calls)
+
+
+    def test_buildProtocolOnce(self):
+        """
+        A L{ftp.DTPFactory} instance's C{buildProtocol} method can be used once
+        to create a L{ftp.DTP} instance.
+        """
+        protocol = self.factory.buildProtocol(None)
+        self.assertIsInstance(protocol, ftp.DTP)
+
+        # A subsequent call returns None.
+        self.assertIdentical(self.factory.buildProtocol(None), None)
+
+
+    def test_timeoutAfterConnection(self):
+        """
+        If a timeout has been set up using L{ftp.DTPFactory.setTimeout}, it is
+        cancelled by L{ftp.DTPFactory.buildProtocol}.
+        """
+        self.factory.setTimeout(10)
+        protocol = self.factory.buildProtocol(None)
+        # Make sure the call is no longer active.
+        self.assertFalse(self.reactor.calls)
+
+
+    def test_connectionAfterTimeout(self):
+        """
+        If L{ftp.DTPFactory.buildProtocol} is called after the timeout
+        specified by L{ftp.DTPFactory.setTimeout} has elapsed, C{None} is
+        returned.
+        """
+        # Handle the error so it doesn't get logged.
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+
+        # Set up the timeout and then cause it to elapse so the Deferred does
+        # fail.
+        self.factory.setTimeout(10)
+        self.reactor.advance(10)
+
+        # Try to get a protocol - we should not be able to.
+        self.assertIdentical(self.factory.buildProtocol(None), None)
+
+        # Make sure the Deferred is doing the right thing.
+        return d
+
+
+    def test_timeoutAfterConnectionFailed(self):
+        """
+        L{ftp.DTPFactory.deferred} fails with L{PortConnectionError} when
+        L{ftp.DTPFactory.clientConnectionFailed} is called.  If the timeout
+        specified with L{ftp.DTPFactory.setTimeout} expires after that, nothing
+        additional happens.
+        """
+        finished = []
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+        d.addCallback(finished.append)
+
+        self.factory.setTimeout(10)
+        self.assertFalse(finished)
+        self.factory.clientConnectionFailed(None, None)
+        self.assertTrue(finished)
+        self.reactor.advance(10)
+        return d
+
+
+    def test_connectionFailedAfterTimeout(self):
+        """
+        If L{ftp.DTPFactory.clientConnectionFailed} is called after the timeout
+        specified by L{ftp.DTPFactory.setTimeout} has elapsed, nothing beyond
+        the normal timeout before happens.
+        """
+        # Handle the error so it doesn't get logged.
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+
+        # Set up the timeout and then cause it to elapse so the Deferred does
+        # fail.
+        self.factory.setTimeout(10)
+        self.reactor.advance(10)
+
+        # Now fail the connection attempt.  This should do nothing.  In
+        # particular, it should not raise an exception.
+        self.factory.clientConnectionFailed(None, defer.TimeoutError("foo"))
+
+        # Give the Deferred to trial so it can make sure it did what we
+        # expected.
+        return d
+
+
+
 # -- Client Tests -----------------------------------------------------------
 
 class PrintLines(protocol.Protocol):
@@ -1638,16 +1838,6 @@ class FTPClientTestCase(unittest.TestCase):
 
 
 
-class DummyTransport:
-    def write(self, bytes):
-        pass
-
-class BufferingTransport:
-    buffer = ''
-    def write(self, bytes):
-        self.buffer += bytes
-
-
 class FTPClientBasicTests(unittest.TestCase):
 
     def testGreeting(self):
@@ -1665,7 +1855,7 @@ class FTPClientBasicTests(unittest.TestCase):
 
     def testMultilineResponse(self):
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = DummyTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Imaginary FTP.')
 
         # Queue (and send) a dummy command, and set up a callback to capture the
@@ -1710,44 +1900,51 @@ class FTPClientBasicTests(unittest.TestCase):
              '321',
              '210 Done.'], result[0])
 
-    def testNoPasswordGiven(self):
-        """Passing None as the password avoids sending the PASS command."""
+
+    def test_noPasswordGiven(self):
+        """
+        Passing None as the password avoids sending the PASS command.
+        """
         # Create a client, and give it a greeting.
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = BufferingTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Welcome to Imaginary FTP.')
 
         # Queue a login with no password
         ftpClient.queueLogin('bob', None)
-        self.failUnlessEqual('USER bob\r\n', ftpClient.transport.buffer)
+        self.assertEquals('USER bob\r\n', ftpClient.transport.value())
 
         # Clear the test buffer, acknowledge the USER command.
-        ftpClient.transport.buffer = ''
+        ftpClient.transport.clear()
         ftpClient.lineReceived('200 Hello bob.')
 
         # The client shouldn't have sent anything more (i.e. it shouldn't have
         # sent a PASS command).
-        self.failUnlessEqual('', ftpClient.transport.buffer)
+        self.assertEquals('', ftpClient.transport.value())
 
-    def testNoPasswordNeeded(self):
-        """Receiving a 230 response to USER prevents PASS from being sent."""
+
+    def test_noPasswordNeeded(self):
+        """
+        Receiving a 230 response to USER prevents PASS from being sent.
+        """
         # Create a client, and give it a greeting.
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = BufferingTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Welcome to Imaginary FTP.')
 
         # Queue a login with no password
         ftpClient.queueLogin('bob', 'secret')
-        self.failUnlessEqual('USER bob\r\n', ftpClient.transport.buffer)
+        self.assertEquals('USER bob\r\n', ftpClient.transport.value())
 
         # Clear the test buffer, acknowledge the USER command with a 230
         # response code.
-        ftpClient.transport.buffer = ''
+        ftpClient.transport.clear()
         ftpClient.lineReceived('230 Hello bob.  No password needed.')
 
         # The client shouldn't have sent anything more (i.e. it shouldn't have
         # sent a PASS command).
-        self.failUnlessEqual('', ftpClient.transport.buffer)
+        self.assertEquals('', ftpClient.transport.value())
+
 
 
 class PathHandling(unittest.TestCase):
@@ -2420,18 +2617,21 @@ class IReadWriteTestsMixin:
 
     def test_write(self):
         """
-        Test L{ftp.IWriteFile}: the implementation should have a receive method
-        returning a C{Deferred} with fires with a consumer ready to receive
-        data to be written.
+        Test L{ftp.IWriteFile}: the implementation should have a receive
+        method returning a C{Deferred} which fires with a consumer ready to
+        receive data to be written. It should also have a close() method that
+        returns a Deferred.
         """
         content = 'elbbow\n'
         def cbGet(writer):
-            return writer.receive().addCallback(cbReceive)
-        def cbReceive(consumer):
+            return writer.receive().addCallback(cbReceive, writer)
+        def cbReceive(consumer, writer):
             producer = TestProducer(content, consumer)
             consumer.registerProducer(None, True)
             producer.start()
             consumer.unregisterProducer()
+            return writer.close().addCallback(cbClose)
+        def cbClose(ignored):
             self.assertEquals(self.getFileContent(), content)
         return self.getFileWriter().addCallback(cbGet)
 
@@ -2473,3 +2673,55 @@ class FTPReadWriteTestCase(unittest.TestCase, IReadWriteTestsMixin):
         Return the content of the temporary file.
         """
         return self.root.child(self.filename).getContent()
+
+
+class CloseTestWriter:
+    implements(ftp.IWriteFile)
+    closeStarted = False
+    def receive(self):
+        self.s = StringIO()
+        fc = ftp.FileConsumer(self.s)
+        return defer.succeed(fc)
+    def close(self):
+        self.closeStarted = True
+        return self.d
+
+class CloseTestShell:
+    def openForWriting(self, segs):
+        return defer.succeed(self.writer)
+
+class FTPCloseTest(unittest.TestCase):
+    """Tests that the server invokes IWriteFile.close"""
+
+    def test_write(self):
+        """Confirm that FTP uploads (i.e. ftp_STOR) correctly call and wait
+        upon the IWriteFile object's close() method"""
+        f = ftp.FTP()
+        f.workingDirectory = ["root"]
+        f.shell = CloseTestShell()
+        f.shell.writer = CloseTestWriter()
+        f.shell.writer.d = defer.Deferred()
+        f.factory = ftp.FTPFactory()
+        f.factory.timeOut = None
+        f.makeConnection(StringIO())
+
+        di = ftp.DTP()
+        di.factory = ftp.DTPFactory(f)
+        f.dtpInstance = di
+        di.makeConnection(None)#
+
+        stor_done = []
+        d = f.ftp_STOR("path")
+        d.addCallback(stor_done.append)
+        # the writer is still receiving data
+        self.assertFalse(f.shell.writer.closeStarted, "close() called early")
+        di.dataReceived("some data here")
+        self.assertFalse(f.shell.writer.closeStarted, "close() called early")
+        di.connectionLost("reason is ignored")
+        # now we should be waiting in close()
+        self.assertTrue(f.shell.writer.closeStarted, "close() not called")
+        self.assertFalse(stor_done)
+        f.shell.writer.d.callback("allow close() to finish")
+        self.assertTrue(stor_done)
+
+        return d # just in case an errback occurred
