@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -7,27 +7,28 @@ Test running processes.
 
 import gzip
 import os
-import popen2
 import sys
 import signal
 import StringIO
 import errno
 import gc
+import stat
 try:
     import fcntl
 except ImportError:
-    fcntl = None
+    fcntl = process = None
+else:
+    from twisted.internet import process
+
 
 from zope.interface.verify import verifyObject
 
+from twisted.python.log import msg
 from twisted.internet import reactor, protocol, error, interfaces, defer
 from twisted.trial import unittest
 from twisted.python import util, runtime, procutils
+from twisted.python.compat import set
 
-try:
-    from twisted.internet import process
-except ImportError:
-    process = None
 
 
 class StubProcessProtocol(protocol.ProcessProtocol):
@@ -252,32 +253,76 @@ class EchoProtocol(protocol.ProcessProtocol):
         self.onEnded.callback(self)
 
 
+
 class SignalProtocol(protocol.ProcessProtocol):
+    """
+    A process protocol that sends a signal when data is first received.
+
+    @ivar deferred: deferred firing on C{processEnded}.
+    @type deferred: L{defer.Deferred}
+
+    @ivar signal: the signal to send to the process.
+    @type signal: C{str}
+
+    @ivar signaled: A flag tracking whether the signal has been sent to the
+        child or not yet.  C{False} until it is sent, then C{True}.
+    @type signaled: C{bool}
+    """
+
     def __init__(self, deferred, sig):
         self.deferred = deferred
         self.signal = sig
+        self.signaled = False
+
 
     def outReceived(self, data):
-        self.transport.signalProcess(self.signal)
+        """
+        Handle the first output from the child process (which indicates it
+        is set up and ready to receive the signal) by sending the signal to
+        it.  Also log all output to help with debugging.
+        """
+        msg("Received %r from child stdout" % (data,))
+        if not self.signaled:
+            self.signaled = True
+            self.transport.signalProcess(self.signal)
+
+
+    def errReceived(self, data):
+        """
+        Log all data received from the child's stderr to help with
+        debugging.
+        """
+        msg("Received %r from child stderr" % (data,))
+
 
     def processEnded(self, reason):
+        """
+        Callback C{self.deferred} with C{None} if C{reason} is a
+        L{error.ProcessTerminated} failure with C{exitCode} set to C{None},
+        C{signal} set to C{self.signal}, and C{status} holding the status code
+        of the exited process. Otherwise, errback with a C{ValueError}
+        describing the problem.
+        """
+        msg("Child exited: %r" % (reason.getTraceback(),))
         if not reason.check(error.ProcessTerminated):
-            self.deferred.callback("wrong termination: %s" % reason)
-            return
+            return self.deferred.errback(
+                ValueError("wrong termination: %s" % (reason,)))
         v = reason.value
+        if isinstance(self.signal, str):
+            signalValue = getattr(signal, 'SIG' + self.signal)
+        else:
+            signalValue = self.signal
         if v.exitCode is not None:
-            self.deferred.callback("SIG%s: exitCode is %s, not None" %
-                                   (self.signal, v.exitCode))
-            return
-        if v.signal != getattr(signal,'SIG'+self.signal):
-            self.deferred.callback("SIG%s: .signal was %s, wanted %s" %
-                                   (self.signal, v.signal,
-                                    getattr(signal,'SIG'+self.signal)))
-            return
-        if os.WTERMSIG(v.status) != getattr(signal,'SIG'+self.signal):
-            self.deferred.callback('SIG%s: %s'
-                                   % (self.signal, os.WTERMSIG(v.status)))
-            return
+            return self.deferred.errback(
+                ValueError("SIG%s: exitCode is %s, not None" %
+                           (self.signal, v.exitCode)))
+        if v.signal != signalValue:
+            return self.deferred.errback(
+                ValueError("SIG%s: .signal was %s, wanted %s" %
+                           (self.signal, v.signal, signalValue)))
+        if os.WTERMSIG(v.status) != signalValue:
+            return self.deferred.errback(
+                ValueError('SIG%s: %s' % (self.signal, os.WTERMSIG(v.status))))
         self.deferred.callback(None)
 
 
@@ -456,6 +501,26 @@ class ProcessTestCase(unittest.TestCase):
         return d.addCallback(processEnded)
 
 
+    def test_unsetPid(self):
+        """
+        Test if pid is None/non-None before/after process termination.  This
+        reuses process_echoer.py to get a process that blocks on stdin.
+        """
+        finished = defer.Deferred()
+        p = TrivialProcessProtocol(finished)
+        exe = sys.executable
+        scriptPath = util.sibpath(__file__, "process_echoer.py")
+        procTrans = reactor.spawnProcess(p, exe,
+                                    [exe, scriptPath], env=None)
+        self.failUnless(procTrans.pid)
+
+        def afterProcessEnd(ignored):
+            self.assertEqual(procTrans.pid, None)
+
+        p.transport.closeStdin()
+        return finished.addCallback(afterProcessEnd)
+
+
     def test_process(self):
         """
         Test running a process: check its output, it exitCode, some property of
@@ -511,13 +576,19 @@ class ProcessTestCase(unittest.TestCase):
         deferredList.addCallback(_check, protocols)
         return deferredList
 
-    def testEcho(self):
+
+    def test_echo(self):
+        """
+        A spawning a subprocess which echoes its stdin to its stdout via
+        C{reactor.spawnProcess} will result in that echoed output being
+        delivered to outReceived.
+        """
         finished = defer.Deferred()
         p = EchoProtocol(finished)
 
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_echoer.py")
-        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
+        reactor.spawnProcess(p, exe, [exe, scriptPath], env=None)
 
         def asserts(ignored):
             self.failIf(p.failure, p.failure)
@@ -529,9 +600,6 @@ class ProcessTestCase(unittest.TestCase):
             return err
 
         return finished.addCallback(asserts).addErrback(takedownProcess)
-    testEcho.timeout = 60 # XXX This should not be.  There is already a
-                          # global timeout value.  Why do you think this
-                          # test can complete more quickly?
 
 
     def testCommandLine(self):
@@ -953,13 +1021,19 @@ class PosixProcessBase:
         d.addCallback(check)
         return d
 
-    def testAbnormalTermination(self):
-        cmd = self.getCommand('false')
+
+    def test_abnormalTermination(self):
+        """
+        When a process terminates with a system exit code set to 1,
+        C{processEnded} is called with a L{error.ProcessTerminated} error,
+        the C{exitCode} attribute reflecting the system exit code.
+        """
+        exe = sys.executable
 
         d = defer.Deferred()
         p = TrivialProcessProtocol(d)
-        reactor.spawnProcess(p, cmd, ['false'], env=None,
-                             usePTY=self.usePTY)
+        reactor.spawnProcess(p, exe, [exe, '-c', 'import sys; sys.exit(1)'],
+                             env=None, usePTY=self.usePTY)
 
         def check(ignored):
             p.reason.trap(error.ProcessTerminated)
@@ -968,30 +1042,87 @@ class PosixProcessBase:
         d.addCallback(check)
         return d
 
+
     def _testSignal(self, sig):
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_signal.py")
         d = defer.Deferred()
         p = SignalProtocol(d, sig)
-        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath, sig],
-                             env=None,
+        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None,
                              usePTY=self.usePTY)
         return d
 
-    def testSignalHUP(self):
-        d = self._testSignal('HUP')
-        d.addCallback(self.failIf)
-        return d
 
-    def testSignalINT(self):
-        d = self._testSignal('INT')
-        d.addCallback(self.failIf)
-        return d
+    def test_signalHUP(self):
+        """
+        Sending the SIGHUP signal to a running process interrupts it, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} set to C{None} and the C{signal} attribute set to
+        C{signal.SIGHUP}. C{os.WTERMSIG} can also be used on the C{status}
+        attribute to extract the signal value.
+        """
+        return self._testSignal('HUP')
+
+
+    def test_signalINT(self):
+        """
+        Sending the SIGINT signal to a running process interrupts it, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} set to C{None} and the C{signal} attribute set to
+        C{signal.SIGINT}. C{os.WTERMSIG} can also be used on the C{status}
+        attribute to extract the signal value.
+        """
+        return self._testSignal('INT')
+
+
+    def test_signalKILL(self):
+        """
+        Sending the SIGKILL signal to a running process interrupts it, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} set to C{None} and the C{signal} attribute set to
+        C{signal.SIGKILL}. C{os.WTERMSIG} can also be used on the C{status}
+        attribute to extract the signal value.
+        """
+        return self._testSignal('KILL')
+
+
+    def test_signalTERM(self):
+        """
+        Sending the SIGTERM signal to a running process interrupts it, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} set to C{None} and the C{signal} attribute set to
+        C{signal.SIGTERM}. C{os.WTERMSIG} can also be used on the C{status}
+        attribute to extract the signal value.
+        """
+        return self._testSignal('TERM')
+
+
+    def test_childSignalHandling(self):
+        """
+        The disposition of signals which are ignored in the parent
+        process is reset to the default behavior for the child
+        process.
+        """
+        # Somewhat arbitrarily select SIGUSR1 here.  It satisfies our
+        # requirements that:
+        #    - The interpreter not fiddle around with the handler
+        #      behind our backs at startup time (this disqualifies
+        #      signals like SIGINT and SIGPIPE).
+        #    - The default behavior is to exit.
+        #
+        # This lets us send the signal to the child and then verify
+        # that it exits with a status code indicating that it was
+        # indeed the signal which caused it to exit.
+        which = signal.SIGUSR1
+
+        # Ignore the signal in the parent (and make sure we clean it
+        # up).
+        handler = signal.signal(which, signal.SIG_IGN)
+        self.addCleanup(signal.signal, signal.SIGUSR1, handler)
+
+        # Now do the test.
+        return self._testSignal(signal.SIGUSR1)
 
-    def testSignalKILL(self):
-        d = self._testSignal('KILL')
-        d.addCallback(self.failIf)
-        return d
 
     def test_executionError(self):
         """
@@ -1019,6 +1150,68 @@ class PosixProcessBase:
         return d
 
 
+    def test_errorInProcessEnded(self):
+        """
+        The handler which reaps a process is removed when the process is
+        reaped, even if the protocol's C{processEnded} method raises an
+        exception.
+        """
+        connected = defer.Deferred()
+        ended = defer.Deferred()
+
+        # This script runs until we disconnect its transport.
+        pythonExecutable = sys.executable
+        scriptPath = util.sibpath(__file__, "process_twisted.py")
+
+        class ErrorInProcessEnded(protocol.ProcessProtocol):
+            """
+            A protocol that raises an error in C{processEnded}.
+            """
+            def makeConnection(self, transport):
+                connected.callback(transport)
+
+            def processEnded(self, reason):
+                reactor.callLater(0, ended.callback, None)
+                raise RuntimeError("Deliberate error")
+
+        # Launch the process.
+        reactor.spawnProcess(
+            ErrorInProcessEnded(), pythonExecutable,
+            [pythonExecutable, scriptPath],
+            env=None, path=None)
+
+        pid = []
+        def cbConnected(transport):
+            pid.append(transport.pid)
+            # There's now a reap process handler registered.
+            self.assertIn(transport.pid, process.reapProcessHandlers)
+
+            # Kill the process cleanly, triggering an error in the protocol.
+            transport.loseConnection()
+        connected.addCallback(cbConnected)
+
+        def checkTerminated(ignored):
+            # The exception was logged.
+            excs = self.flushLoggedErrors(RuntimeError)
+            self.assertEqual(len(excs), 1)
+            # The process is no longer scheduled for reaping.
+            self.assertNotIn(pid[0], process.reapProcessHandlers)
+        ended.addCallback(checkTerminated)
+
+        return ended
+
+
+
+class MockSignal(object):
+    """
+    Neuter L{signal.signal}, but pass other attributes unscathed
+    """
+    def signal(self, sig, action):
+        return signal.getsignal(sig)
+
+    def __getattr__(self, attr):
+        return getattr(signal, attr)
+
 
 class MockOS(object):
     """
@@ -1066,25 +1259,47 @@ class MockOS(object):
 
     @ivar waitChild: if set, subsequent calls to waitpid will return it.
     @type waitChild: C{None} or a tuple
+
+    @ivar euid: the uid returned by the fake C{os.geteuid}
+    @type euid: C{int}
+
+    @ivar egid: the gid returned by the fake C{os.getegid}
+    @type egid: C{int}
+
+    @ivar seteuidCalls: stored results of C{os.seteuid} calls.
+    @type seteuidCalls: C{list}
+
+    @ivar setegidCalls: stored results of C{os.setegid} calls.
+    @type setegidCalls: C{list}
+
+    @ivar path: the path returned by C{os.path.expanduser}.
+    @type path: C{str}
     """
     exited = False
-    O_RDWR = 1
-    O_NOCTTY = 1
-    WNOHANG = 1
     raiseExec = False
     fdio = None
     child = True
     raiseWaitPid = None
     raiseFork = None
     waitChild = None
+    euid = 0
+    egid = 0
+    path = None
 
     def __init__(self):
         """
-        Initialiaze data structures.
+        Initialize data structures.
         """
         self.actions = []
         self.closed = []
         self.pipeCount = 0
+        self.O_RDWR = -1
+        self.O_NOCTTY = -2
+        self.WNOHANG = -4
+        self.WEXITSTATUS = lambda x: 0
+        self.WIFEXITED = lambda x: 1
+        self.seteuidCalls = []
+        self.setegidCalls = []
 
 
     def open(self, dev, flags):
@@ -1095,6 +1310,13 @@ class MockOS(object):
         return -3
 
 
+    def fstat(self, fd):
+        """
+        Fake C{os.fstat}.  Return a C{os.stat_result} filled with garbage.
+        """
+        return os.stat_result((0,) * 10)
+
+
     def fdopen(self, fd, flag):
         """
         Fake C{os.fdopen}. Return a StringIO object whose content can be tested
@@ -1214,13 +1436,6 @@ class MockOS(object):
         """
 
 
-    def getegid(self):
-        """
-        Override C{os.getegid}. Return a dumb number.
-        """
-        return 1234
-
-
     def getgid(self):
         """
         Override C{os.getgid}. Return a dumb number.
@@ -1228,13 +1443,6 @@ class MockOS(object):
         return 1235
 
 
-    def geteuid(self):
-        """
-        Override C{os.geteuid}. Return a dumb number.
-        """
-        return 1236
-
-
     def getuid(self):
         """
         Override C{os.getuid}. Return a dumb number.
@@ -1277,6 +1485,55 @@ class MockOS(object):
         self.actions.append(('switchuid', uid, gid))
 
 
+    def openpty(self):
+        """
+        Override C{pty.openpty}, returning fake file descriptors.
+        """
+        return -12, -13
+
+
+    def geteuid(self):
+        """
+        Mock C{os.geteuid}, returning C{self.euid} instead.
+        """
+        return self.euid
+
+
+    def getegid(self):
+        """
+        Mock C{os.getegid}, returning C{self.egid} instead.
+        """
+        return self.egid
+
+
+    def seteuid(self, egid):
+        """
+        Mock C{os.seteuid}, store result.
+        """
+        self.seteuidCalls.append(egid)
+
+
+    def setegid(self, egid):
+        """
+        Mock C{os.setegid}, store result.
+        """
+        self.setegidCalls.append(egid)
+
+
+    def expanduser(self, path):
+        """
+        Mock C{os.path.expanduser}.
+        """
+        return self.path
+
+
+    def getpwnam(self, user):
+        """
+        Mock C{pwd.getpwnam}.
+        """
+        return 0, 0, 1, 2
+
+
 
 if process is not None:
     class DumbProcessWriter(process.ProcessWriter):
@@ -1319,42 +1576,38 @@ class MockProcessTestCase(unittest.TestCase):
     """
     Mock a process runner to test forked child code path.
     """
+    if process is None:
+        skip = "twisted.internet.process is never used on Windows"
 
     def setUp(self):
         """
-        Replace L{process} os, fcntl, sys, switchUID modules with the mock
-        class L{MockOS}.
+        Replace L{process} os, fcntl, sys, switchUID, fdesc and pty modules
+        with the mock class L{MockOS}.
         """
         if gc.isenabled():
             self.addCleanup(gc.enable)
         else:
             self.addCleanup(gc.disable)
         self.mockos = MockOS()
-        self.oldos = os
-        self.oldfcntl = fcntl
-        self.oldsys = sys
-        self.oldSwitchUID = util.switchUID
-        self.oldFdesc = process.fdesc
-        process.os = self.mockos
-        process.fcntl = self.mockos
-        process.sys = self.mockos
-        process.switchUID = self.mockos.switchUID
-        process.fdesc = self.mockos
-        process.Process.processReaderFactory = DumbProcessReader
-        process.Process.processWriterFactory = DumbProcessWriter
+        self.mockos.euid = 1236
+        self.mockos.egid = 1234
+        self.patch(process, "os", self.mockos)
+        self.patch(process, "fcntl", self.mockos)
+        self.patch(process, "sys", self.mockos)
+        self.patch(process, "switchUID", self.mockos.switchUID)
+        self.patch(process, "fdesc", self.mockos)
+        self.patch(process.Process, "processReaderFactory", DumbProcessReader)
+        self.patch(process.Process, "processWriterFactory", DumbProcessWriter)
+        self.patch(process, "pty", self.mockos)
+
+        self.mocksig = MockSignal()
+        self.patch(process, "signal", self.mocksig)
 
 
     def tearDown(self):
         """
-        Restore L{process} modules, and reset processes registered for reap.
+        Reset processes registered for reap.
         """
-        process.os = self.oldos
-        process.fcntl = self.oldfcntl
-        process.sys = self.oldsys
-        process.switchUID = self.oldSwitchUID
-        process.fdesc = self.oldFdesc
-        process.Process.processReaderFactory = process.ProcessReader
-        process.Process.processWriterFactory = process.ProcessWriter
         process.reapProcessHandlers = {}
 
 
@@ -1397,7 +1650,7 @@ class MockProcessTestCase(unittest.TestCase):
         reactor.spawnProcess(p, cmd, ['ouch'], env=None,
                              usePTY=False)
         # It should close the first read pipe, and the 2 last write pipes
-        self.assertEqual(self.mockos.closed, [-1, -4, -6])
+        self.assertEqual(set(self.mockos.closed), set([-1, -4, -6]))
         self.assertEquals(self.mockos.actions, [("fork", False), "waitpid"])
 
 
@@ -1477,6 +1730,62 @@ class MockProcessTestCase(unittest.TestCase):
         self.assertFalse(gc.isenabled())
 
 
+    def test_mockForkErrorCloseFDs(self):
+        """
+        When C{os.fork} raises an exception, the file descriptors created
+        before are closed and don't leak.
+        """
+        self._mockWithForkError()
+        self.assertEqual(set(self.mockos.closed), set([-1, -4, -6, -2, -3, -5]))
+
+
+    def test_mockForkErrorGivenFDs(self):
+        """
+        When C{os.forks} raises an exception and that file descriptors have
+        been specified with the C{childFDs} arguments of
+        L{reactor.spawnProcess}, they are not closed.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+            childFDs={0: -10, 1: -11, 2: -13})
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(self.mockos.closed, [])
+
+        # We can also put "r" or "w" to let twisted create the pipes
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+            childFDs={0: "r", 1: -11, 2: -13})
+        self.assertEqual(set(self.mockos.closed), set([-1, -2]))
+
+
+    def test_mockForkErrorClosePTY(self):
+        """
+        When C{os.fork} raises an exception, the file descriptors created by
+        C{pty.openpty} are closed and don't leak, when C{usePTY} is set to
+        C{True}.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          usePTY=True)
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(set(self.mockos.closed), set([-12, -13]))
+
+
+    def test_mockForkErrorPTYGivenFDs(self):
+        """
+        If a tuple is passed to C{usePTY} to specify slave and master file
+        descriptors and that C{os.fork} raises an exception, these file
+        descriptors aren't closed.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          usePTY=(-20, -21, 'foo'))
+        self.assertEqual(self.mockos.actions, [("fork", False)])
+        self.assertEqual(self.mockos.closed, [])
+
+
     def test_mockWithExecError(self):
         """
         Spawn a process but simulate an error during execution in the client
@@ -1629,25 +1938,65 @@ class MockProcessTestCase(unittest.TestCase):
         proc.reapProcess()
 
 
-class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
-    # add three non-pty test cases
+    def test_mockErrorInPipe(self):
+        """
+        If C{os.pipe} raises an exception after some pipes where created, the
+        created pipes are closed and don't leak.
+        """
+        pipes = [-1, -2, -3, -4]
+        def pipe():
+            try:
+                return pipes.pop(0), pipes.pop(0)
+            except IndexError:
+                raise OSError()
+        self.mockos.pipe = pipe
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None)
+        self.assertEqual(self.mockos.actions, [])
+        self.assertEqual(set(self.mockos.closed), set([-4, -3, -2, -1]))
 
-    def testStderr(self):
-        # we assume there is no file named ZZXXX..., both in . and in /tmp
-        cmd = self.getCommand('ls')
+
+    def test_mockErrorInForkRestoreUID(self):
+        """
+        If C{os.fork} raises an exception and a UID change has been made, the
+        previous UID and GID are restored.
+        """
+        self.mockos.raiseFork = OSError(errno.EAGAIN, None)
+        protocol = TrivialProcessProtocol(None)
+        self.assertRaises(OSError, reactor.spawnProcess, protocol, None,
+                          uid=8080)
+        self.assertEqual(self.mockos.actions,
+            [('setuid', 0), ('setgid', 0), ("fork", False),
+             ('setregid', 1235, 1234), ('setreuid', 1237, 1236)])
+
+
+
+class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
+    # add two non-pty test cases
+
+    def test_stderr(self):
+        """
+        Bytes written to stderr by the spawned process are passed to the
+        C{errReceived} callback on the C{ProcessProtocol} passed to
+        C{spawnProcess}.
+        """
+        cmd = sys.executable
+
+        value = "42"
 
         p = Accumulator()
         d = p.endedDeferred = defer.Deferred()
         reactor.spawnProcess(p, cmd,
-                             [cmd,
-                              "ZZXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"],
+                             [cmd, "-c", 
+                              "import sys; sys.stderr.write('%s')" % (value,)],
                              env=None, path="/tmp",
                              usePTY=self.usePTY)
 
         def processEnded(ign):
-            self.assertEquals(lsOut, p.errF.getvalue())
+            self.assertEquals(value, p.errF.getvalue())
         return d.addCallback(processEnded)
 
+
     def testProcess(self):
         cmd = self.getCommand('gzip')
         s = "there's no place like home!\n" * 3
@@ -1674,7 +2023,7 @@ class PosixProcessTestCasePTY(unittest.TestCase, PosixProcessBase):
     usePTY = True
     # PTYs only offer one input and one output. What still makes sense?
     # testNormalTermination
-    # testAbnormalTermination
+    # test_abnormalTermination
     # testSignal
     # testProcess, but not without p.transport.closeStdin
     #  might be solveable: TODO: add test if so
@@ -1705,6 +2054,31 @@ class PosixProcessTestCasePTY(unittest.TestCase, PosixProcessBase):
         self.assertRaises(ValueError, reactor.spawnProcess, p, pyExe, pyArgs,
             usePTY=1, childFDs={1:'r'})
 
+
+
+class Win32SignalProtocol(SignalProtocol):
+    """
+    A win32-specific process protocol that handles C{processEnded}
+    differently: processes should exit with exit code 1.
+    """
+
+    def processEnded(self, reason):
+        """
+        Callback C{self.deferred} with C{None} if C{reason} is a
+        L{error.ProcessTerminated} failure with C{exitCode} set to 1.
+        Otherwise, errback with a C{ValueError} describing the problem.
+        """
+        if not reason.check(error.ProcessTerminated):
+            return self.deferred.errback(
+                ValueError("wrong termination: %s" % (reason,)))
+        v = reason.value
+        if v.exitCode != 1:
+            return self.deferred.errback(
+                ValueError("Wrong exit code: %s" % (reason.exitCode,)))
+        self.deferred.callback(None)
+
+
+
 class Win32ProcessTestCase(unittest.TestCase):
     """
     Test process programs that are packaged with twisted.
@@ -1740,6 +2114,129 @@ class Win32ProcessTestCase(unittest.TestCase):
             reactor.spawnProcess, p, pyExe, pyArgs, childFDs={1:'r'})
 
 
+    def _testSignal(self, sig):
+        exe = sys.executable
+        scriptPath = util.sibpath(__file__, "process_signal.py")
+        d = defer.Deferred()
+        p = Win32SignalProtocol(d, sig)
+        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
+        return d
+
+
+    def test_signalTERM(self):
+        """
+        Sending the SIGTERM signal terminates a created process, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} attribute set to 1.
+        """
+        return self._testSignal('TERM')
+
+
+    def test_signalINT(self):
+        """
+        Sending the SIGINT signal terminates a created process, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} attribute set to 1.
+        """
+        return self._testSignal('INT')
+
+
+    def test_signalKILL(self):
+        """
+        Sending the SIGKILL signal terminates a created process, and
+        C{processEnded} is called with a L{error.ProcessTerminated} instance
+        with the C{exitCode} attribute set to 1.
+        """
+        return self._testSignal('KILL')
+
+
+    def test_closeHandles(self):
+        """
+        The win32 handles should be properly closed when the process exits.
+        """
+        import win32api
+
+        connected = defer.Deferred()
+        ended = defer.Deferred()
+
+        class SimpleProtocol(protocol.ProcessProtocol):
+            """
+            A protocol that fires deferreds when connected and disconnected.
+            """
+            def makeConnection(self, transport):
+                connected.callback(transport)
+
+            def processEnded(self, reason):
+                ended.callback(None)
+
+        p = SimpleProtocol()
+
+        pyExe = sys.executable
+        pyArgs = [pyExe, "-u", "-c", "print 'hello'"]
+        proc = reactor.spawnProcess(p, pyExe, pyArgs)
+
+        def cbConnected(transport):
+            self.assertIdentical(transport, proc)
+            # perform a basic validity test on the handles
+            win32api.GetHandleInformation(proc.hProcess)
+            win32api.GetHandleInformation(proc.hThread)
+            # And save their values for later
+            self.hProcess = proc.hProcess
+            self.hThread = proc.hThread
+        connected.addCallback(cbConnected)
+
+        def checkTerminated(ignored):
+            # The attributes on the process object must be reset...
+            self.assertIdentical(proc.pid, None)
+            self.assertIdentical(proc.hProcess, None)
+            self.assertIdentical(proc.hThread, None)
+            # ...and the handles must be closed.
+            self.assertRaises(win32api.error,
+                              win32api.GetHandleInformation, self.hProcess)
+            self.assertRaises(win32api.error,
+                              win32api.GetHandleInformation, self.hThread)
+        ended.addCallback(checkTerminated)
+
+        return defer.gatherResults([connected, ended])
+
+
+
+class Dumbwin32procPidTest(unittest.TestCase):
+    """
+    Simple test for the pid attribute of Process on win32.
+    """
+
+    def test_pid(self):
+        """
+        Launch process with mock win32process. The only mock aspect of this
+        module is that the pid of the process created will always be 42.
+        """
+        from twisted.internet import _dumbwin32proc
+        from twisted.test import mock_win32process
+        self.patch(_dumbwin32proc, "win32process", mock_win32process)
+        exe = sys.executable
+        scriptPath = util.sibpath(__file__, "process_cmdline.py")
+
+        d = defer.Deferred()
+        processProto = TrivialProcessProtocol(d)
+        comspec = str(os.environ["COMSPEC"])
+        cmd = [comspec, "/c", exe, scriptPath]
+
+        p = _dumbwin32proc.Process(reactor,
+                                  processProto,
+                                  None,
+                                  cmd,
+                                  {},
+                                  None)
+        self.assertEquals(42, p.pid)
+        self.assertEquals("<Process pid=42>", repr(p))
+
+        def pidCompleteCb(result):
+            self.assertEquals(None, p.pid)
+        return d.addCallback(pidCompleteCb)
+
+
+
 class UtilTestCase(unittest.TestCase):
     """
     Tests for process-related helper functions (currently only
@@ -1780,8 +2277,10 @@ class UtilTestCase(unittest.TestCase):
 
     def tearDown(self):
         """
-        Restore the saved PATH setting.
+        Restore the saved PATH setting, and set all created files readable
+        again so that they can be deleted easily.
         """
+        os.chmod(os.path.join(self.bazbar, "executable"), stat.S_IWUSR)
         if self.oldPath is None:
             try:
                 del os.environ['PATH']
@@ -1909,17 +2408,13 @@ if (runtime.platform.getType() != 'posix') or (not interfaces.IReactorProcess(re
     PosixProcessTestCasePTY.skip = skipMessage
     TestTwoProcessesPosix.skip = skipMessage
     FDTest.skip = skipMessage
-else:
-    # do this before running the tests: it uses SIGCHLD and stuff internally
-    lsOut = popen2.popen3("/bin/ls ZZXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX")[2].read()
 
 if (runtime.platform.getType() != 'win32') or (not interfaces.IReactorProcess(reactor, None)):
     Win32ProcessTestCase.skip = skipMessage
     TestTwoProcessesNonPosix.skip = skipMessage
+    Dumbwin32procPidTest.skip = skipMessage
 
 if not interfaces.IReactorProcess(reactor, None):
     ProcessTestCase.skip = skipMessage
     ClosingPipes.skip = skipMessage
 
-if process is None:
-    MockProcessTestCase.skip = skipMessage
