@@ -1,8 +1,8 @@
-# Copyright (c) 2007-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
-Tests for L{strcred}.
+Tests for L{twisted.cred.strcred}.
 """
 
 import os
@@ -13,7 +13,8 @@ from twisted.trial import unittest
 from twisted.cred import credentials, checkers, error, strcred
 from twisted.plugins import cred_file, cred_anonymous
 from twisted.python import usage
-
+from twisted.python.filepath import FilePath
+from twisted.python.fakepwd import UserDatabase
 
 try:
     import crypt
@@ -174,11 +175,6 @@ class TestUnixChecker(unittest.TestCase):
         }
 
 
-    def _pwd(self, username):
-        return (username, crypt.crypt(self.users[username], 'F/'),
-                1000, 1000, username, '/home/'+username, '/bin/sh')
-
-
     def _spwd(self, username):
         return (username, crypt.crypt(self.users[username], 'F/'),
                 0, 0, 99999, 7, -1, -1, -1)
@@ -190,19 +186,22 @@ class TestUnixChecker(unittest.TestCase):
         self.badPass = credentials.UsernamePassword('alice', 'foobar')
         self.badUser = credentials.UsernamePassword('x', 'yz')
         self.checker = strcred.makeChecker('unix')
+
         # Hack around the pwd and spwd modules, since we can't really
         # go about reading your /etc/passwd or /etc/shadow files
         if pwd:
-            self._pwd_getpwnam = pwd.getpwnam
-            pwd.getpwnam = self._pwd
+            database = UserDatabase()
+            for username, password in self.users.items():
+                database.addUser(
+                    username, crypt.crypt(password, 'F/'),
+                    1000, 1000, username, '/home/' + username, '/bin/sh')
+            self.patch(pwd, 'getpwnam', database.getpwnam)
         if spwd:
             self._spwd_getspnam = spwd.getspnam
             spwd.getspnam = self._spwd
 
 
     def tearDown(self):
-        if pwd:
-            pwd.getpwnam = self._pwd_getpwnam
         if spwd:
             spwd.getspnam = self._spwd_getspnam
 
@@ -245,10 +244,15 @@ class TestUnixChecker(unittest.TestCase):
 
 
     if None in (pwd, spwd, crypt):
+        availability = []
+        for module, name in ((pwd, "pwd"), (spwd, "swpd"), (crypt, "crypt")):
+            if module is None:
+                availability += [name]
         for method in (test_unixCheckerSucceeds,
                        test_unixCheckerFailsUsername,
                        test_unixCheckerFailsPassword):
-            method.skip = 'pwd and spwd are both unavailable'
+            method.skip = ("Required module(s) are unavailable: " +
+                           ", ".join(availability))
 
 
 
@@ -263,7 +267,7 @@ class TestFileDBChecker(unittest.TestCase):
         self.badPass = credentials.UsernamePassword('alice', 'foobar')
         self.badUser = credentials.UsernamePassword('x', 'yz')
         self.filename = self.mktemp()
-        file(self.filename, 'w').write('admin:asdf\nalice:foo\n')
+        FilePath(self.filename).setContent('admin:asdf\nalice:foo\n')
         self.checker = strcred.makeChecker('file:' + self.filename)
 
 
