@@ -1,17 +1,19 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details
 
-import StringIO, warnings
+"""
+Tests for assertions provided by L{twisted.trial.unittest.TestCase}.
+"""
+
+import warnings
 from pprint import pformat
 
 from twisted.python import reflect, failure
 from twisted.python.deprecate import deprecated, getVersionString
 from twisted.python.versions import Version
-from twisted.python.util import dsu
 from twisted.internet import defer
 from twisted.trial import unittest, runner, reporter
 
-
 class MockEquality(object):
     def __init__(self, name):
         self.name = name
@@ -26,7 +28,8 @@ class MockEquality(object):
 
 
 class TestAssertions(unittest.TestCase):
-    """Tests for TestCase's assertion methods.  That is, failUnless*,
+    """
+    Tests for TestCase's assertion methods.  That is, failUnless*,
     failIf*, assert*.
 
     This is pretty paranoid.  Still, a certain paranoia is healthy if you
@@ -49,7 +52,6 @@ class TestAssertions(unittest.TestCase):
 
     def test_failingException_fails(self):
         test = runner.TestLoader().loadClass(TestAssertions.FailingTest)
-        io = StringIO.StringIO()
         result = reporter.TestResult()
         test.run(result)
         self.failIf(result.wasSuccessful())
@@ -162,7 +164,7 @@ class TestAssertions(unittest.TestCase):
         except TypeError:
             self.fail("failUnlessRaises shouldn't re-raise unexpected "
                       "exceptions")
-        except self.failureException, e:
+        except self.failureException:
             # what we expect
             pass
         else:
@@ -186,7 +188,7 @@ class TestAssertions(unittest.TestCase):
         try:
             x = self.failUnlessRaises(self.failureException, self._raiseError,
                                       ValueError)
-        except self.failureException, e:
+        except self.failureException:
             # what we expect
             pass
         else:
@@ -304,7 +306,8 @@ class TestAssertions(unittest.TestCase):
         return d
 
     def test_assertFailure_moreInfo(self):
-        """In the case of assertFailure failing, check that we get lots of
+        """
+        In the case of assertFailure failing, check that we get lots of
         information about the exception that was raised.
         """
         try:
@@ -323,7 +326,8 @@ class TestAssertions(unittest.TestCase):
         self.assertIn(f.getBriefTraceback(), output)
 
     def test_assertFailure_masked(self):
-        """A single wrong assertFailure should fail the whole test.
+        """
+        A single wrong assertFailure should fail the whole test.
         """
         class ExampleFailure(Exception):
             pass
@@ -406,9 +410,34 @@ class TestAssertions(unittest.TestCase):
             deprecated, 123)
 
 
+    def test_assertWarnsWrongFile(self):
+        """
+        If the warning emitted by a function refers to a different file than is
+        passed to C{assertWarns}, C{failureException} is raised.
+        """
+        def deprecated(a):
+            # stacklevel=2 points at the direct caller of the function.  The
+            # way assertRaises is invoked below, the direct caller will be
+            # something somewhere in trial, not something in this file.  In
+            # Python 2.5 and earlier, stacklevel of 0 resulted in a warning
+            # pointing to the warnings module itself.  Starting in Python 2.6,
+            # stacklevel of 0 and 1 both result in a warning pointing to *this*
+            # file, presumably due to the fact that the warn function is
+            # implemented in C and has no convenient Python
+            # filename/linenumber.
+            warnings.warn(
+                "Foo deprecated", category=DeprecationWarning, stacklevel=2)
+        self.assertRaises(
+            self.failureException,
+            # Since the direct caller isn't in this file, try to assert that
+            # the warning *does* point to this file, so that assertWarns raises
+            # an exception.
+            self.assertWarns, DeprecationWarning, "Foo deprecated", __file__,
+            deprecated, 123)
+
     def test_assertWarnsOnClass(self):
         """
-        Test asserWarns works when creating a class instance.
+        Test assertWarns works when creating a class instance.
         """
         class Warn:
             def __init__(self):
@@ -469,25 +498,16 @@ class TestAssertions(unittest.TestCase):
 
     def test_assertWarnsMultipleWarnings(self):
         """
-        Check that assertWarns is able to handle multiple warnings produced by
-        the same function.
+        C{assertWarns} does not raise an exception if the function it is passed
+        triggers the same warning more than once.
         """
-        i = [0]
-        def deprecated(a):
-            # the stacklevel is important here, because as the function is
-            # recursive, the warnings produced have a different stack, but we
-            # want to be sure that only the first warning is tested against
-            warnings.warn("Woo deprecated",
-                category=PendingDeprecationWarning,
-                stacklevel=2)
-            i[0] += 1
-            if i[0] < 3:
-                return deprecated(a)
-            else:
-                return a
-        r = self.assertWarns(PendingDeprecationWarning, "Woo deprecated",
-            unittest.__file__, deprecated, 123)
-        self.assertEquals(r, 123)
+        def deprecated():
+            warnings.warn("Woo deprecated", category=PendingDeprecationWarning)
+        def f():
+            deprecated()
+            deprecated()
+        self.assertWarns(
+            PendingDeprecationWarning, "Woo deprecated", __file__, f)
 
 
     def test_assertWarnsDifferentWarnings(self):
@@ -504,6 +524,22 @@ class TestAssertions(unittest.TestCase):
         self.assertEquals(str(e), "Can't handle different warnings")
 
 
+    def test_assertWarnsAfterUnassertedWarning(self):
+        """
+        Warnings emitted before L{TestCase.assertWarns} is called do not get
+        flushed and do not alter the behavior of L{TestCase.assertWarns}.
+        """
+        class TheWarning(Warning):
+            pass
+
+        def f(message):
+            warnings.warn(message, category=TheWarning)
+        f("foo")
+        self.assertWarns(TheWarning, "bar", __file__, f, "bar")
+        [warning] = self.flushWarnings([f])
+        self.assertEqual(warning['message'], "foo")
+
+
     def test_assertIsInstance(self):
         """
         Test a true condition of assertIsInstance.
@@ -565,7 +601,9 @@ class TestAssertions(unittest.TestCase):
         """
         A = type('A', (object,), {})
         a = A()
-        self.assertRaises(self.failureException, self.assertNotIsInstance, a, A)
+        error = self.assertRaises(self.failureException,
+                                  self.assertNotIsInstance, a, A)
+        self.assertEquals(str(error), "%r is an instance of %s" % (a, A))
 
     def test_assertNotIsInstanceErrorMultipleClasses(self):
         """
@@ -579,7 +617,8 @@ class TestAssertions(unittest.TestCase):
 
 
 class TestAssertionNames(unittest.TestCase):
-    """Tests for consistency of naming within TestCase assertion methods
+    """
+    Tests for consistency of naming within TestCase assertion methods
     """
     def _getAsserts(self):
         dct = {}
@@ -589,17 +628,27 @@ class TestAssertionNames(unittest.TestCase):
     def _name(self, x):
         return x.__name__
 
-    def test_failUnless_matches_assert(self):
-        asserts = self._getAsserts()
-        failUnlesses = reflect.prefixedMethods(self, 'failUnless')
-        self.failUnlessEqual(dsu(asserts, self._name),
-                             dsu(failUnlesses, self._name))
+
+    def test_failUnlessMatchesAssert(self):
+        """
+        The C{failUnless*} test methods are a subset of the C{assert*} test
+        methods.  This is intended to ensure that methods using the
+        I{failUnless} naming scheme are not added without corresponding methods
+        using the I{assert} naming scheme.  The I{assert} naming scheme is
+        preferred, and new I{assert}-prefixed methods may be added without
+        corresponding I{failUnless}-prefixed methods.
+        """
+        asserts = set(self._getAsserts())
+        failUnlesses = set(reflect.prefixedMethods(self, 'failUnless'))
+        self.assertEquals(
+            failUnlesses, asserts.intersection(failUnlesses))
+
 
     def test_failIf_matches_assertNot(self):
         asserts = reflect.prefixedMethods(unittest.TestCase, 'assertNot')
         failIfs = reflect.prefixedMethods(unittest.TestCase, 'failIf')
-        self.failUnlessEqual(dsu(asserts, self._name),
-                             dsu(failIfs, self._name))
+        self.failUnlessEqual(sorted(asserts, key=self._name),
+                             sorted(failIfs, key=self._name))
 
     def test_equalSpelling(self):
         for name, value in vars(self).items():
@@ -617,7 +666,7 @@ class TestAssertionNames(unittest.TestCase):
 
 class TestCallDeprecated(unittest.TestCase):
     """
-    Test the L{TestCase.callDeprecated} method.
+    Test use of the L{TestCase.callDeprecated} method with version objects.
     """
 
     version = Version('Twisted', 8, 0, 0)
@@ -627,7 +676,6 @@ class TestCallDeprecated(unittest.TestCase):
         Deprecated method for testing.
         """
         return x
-    oldMethod = deprecated(version)(oldMethod)
 
 
     def test_callDeprecatedSuppressesWarning(self):
@@ -635,12 +683,9 @@ class TestCallDeprecated(unittest.TestCase):
         callDeprecated calls a deprecated callable, suppressing the
         deprecation warning.
         """
-        warningsShown = []
-        def warnExplicit(*args):
-            warningsShown.append(args)
-        self.patch(warnings, 'warn_explicit', warnExplicit)
         self.callDeprecated(self.version, self.oldMethod, 'foo')
-        self.assertEqual([], warningsShown, "No warnings should be shown")
+        self.assertEqual(
+            self.flushWarnings(), [], "No warnings should be shown")
 
 
     def test_callDeprecatedCallsFunction(self):
@@ -694,3 +739,12 @@ class TestCallDeprecated(unittest.TestCase):
         nestedDeprecation = deprecated(differentVersion)(nestedDeprecation)
 
         self.callDeprecated(differentVersion, nestedDeprecation, 24)
+
+        # The oldMethod deprecation should have been emitted too, not captured
+        # by callDeprecated.  Flush it now to make sure it did happen and to
+        # prevent it from showing up on stdout.
+        warningsShown = self.flushWarnings()
+        self.assertEqual(len(warningsShown), 1)
+
+TestCallDeprecated.oldMethod = deprecated(TestCallDeprecated.version)(
+    TestCallDeprecated.oldMethod)
