@@ -1,11 +1,17 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 #
-# Maintainer: Jonathan Lange <jml@twistedmatrix.com>
+# Maintainer: Jonathan Lange
+
+"""
+Tests for L{twisted.trial.reporter}.
+"""
 
 
 import errno, sys, os, re, StringIO
+
 from twisted.internet.utils import suppressWarnings
+from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.trial import itrial, unittest, runner, reporter, util
 from twisted.trial.reporter import UncleanWarningsReporterWrapper
@@ -320,8 +326,15 @@ class PyunitTestNames(unittest.TestCase):
 
     def test_minimalReporter(self):
         """
-        The summary of L{reporter.MinimalReporter} is a simple list of
-        numbers, indicating how many tests ran, how many failed etc.
+        The summary of L{reporter.MinimalReporter} is a simple list of numbers,
+        indicating how many tests ran, how many failed etc.
+
+        The numbers represents:
+         * the run time of the tests
+         * the number of tests run, printed 2 times for legacy reasons
+         * the number of errors
+         * the number of failures
+         * the number of skips
         """
         result = reporter.MinimalReporter(self.stream)
         self.test.run(result)
@@ -330,6 +343,32 @@ class PyunitTestNames(unittest.TestCase):
         self.failUnlessEqual(output[1:], ['1', '1', '0', '0', '0'])
 
 
+    def test_minimalReporterTime(self):
+        """
+        L{reporter.MinimalReporter} reports the time to run the tests as first
+        data in its output.
+        """
+        times = [1.0, 1.2, 1.5, 1.9]
+        result = reporter.MinimalReporter(self.stream)
+        result._getTime = lambda: times.pop(0)
+        self.test.run(result)
+        result._printSummary()
+        output = self.stream.getvalue().strip().split(' ')
+        timer = output[0]
+        self.assertEquals(timer, "0.7")
+
+
+    def test_emptyMinimalReporter(self):
+        """
+        The summary of L{reporter.MinimalReporter} is a list of zeroes when no
+        test is actually run.
+        """
+        result = reporter.MinimalReporter(self.stream)
+        result._printSummary()
+        output = self.stream.getvalue().strip().split(' ')
+        self.failUnlessEqual(output, ['0', '0', '0', '0', '0', '0'])
+
+
 
 class TestDirtyReactor(unittest.TestCase):
     """
@@ -348,7 +387,7 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_errorByDefault(self):
         """
-        C{DirtyReactorAggregateError}s are reported as errors with the default
+        L{DirtyReactorAggregateError}s are reported as errors with the default
         Reporter.
         """
         result = reporter.Reporter(stream=self.output)
@@ -359,8 +398,8 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_warningsEnabled(self):
         """
-        C{DirtyReactorErrors}s are reported as warnings when using the
-        L{UncleanWarningsReporterWrapper}.
+        L{DirtyReactorAggregateError}s are reported as warnings when using
+        the L{UncleanWarningsReporterWrapper}.
         """
         result = UncleanWarningsReporterWrapper(
             reporter.Reporter(stream=self.output))
@@ -371,7 +410,7 @@ class TestDirtyReactor(unittest.TestCase):
 
     def test_warningsMaskErrors(self):
         """
-        C{DirtyReactorErrors}s are I{not} reported as errors if the
+        L{DirtyReactorAggregateError}s are I{not} reported as errors if the
         L{UncleanWarningsReporterWrapper} is used.
         """
         result = UncleanWarningsReporterWrapper(
@@ -387,8 +426,8 @@ class TestDirtyReactor(unittest.TestCase):
         Some annoying stuff can pass three-tuples to addError instead of
         Failures (like PyUnit). The wrapper, of course, handles this case,
         since it is a part of L{twisted.trial.itrial.IReporter}! But it does
-        not convert L{DirtyReactorError} to warnings in this case, because
-        nobody should be passing those in the form of three-tuples.
+        not convert L{DirtyReactorAggregateError} to warnings in this case,
+        because nobody should be passing those in the form of three-tuples.
         """
         result = UncleanWarningsReporterWrapper(
             reporter.Reporter(stream=self.output))
@@ -669,17 +708,16 @@ class MockColorizer:
     """
     Used by TestTreeReporter to make sure that output is colored correctly.
     """
+
     def __init__(self, stream):
         self.log = []
 
-    def supported(self):
-        return True
-    supported = classmethod(supported)
 
     def write(self, text, color):
         self.log.append((color, text))
 
 
+
 class TestTreeReporter(unittest.TestCase):
     def setUp(self):
         from twisted.trial.test import sample
@@ -774,21 +812,99 @@ class TestTreeReporter(unittest.TestCase):
         self.assertEqual([], self.result._getPreludeSegments('foo'))
 
 
-class TestReporter(unittest.TestCase):
+
+class TestReporterInterface(unittest.TestCase):
+    """
+    Tests for the bare interface of a trial reporter.
+
+    Subclass this test case and provide a different 'resultFactory' to test
+    that a particular reporter implementation will work with the rest of
+    Trial.
+
+    @cvar resultFactory: A callable that returns a reporter to be tested. The
+        callable must take the same parameters as L{reporter.Reporter}.
+    """
+
     resultFactory = reporter.Reporter
 
     def setUp(self):
         from twisted.trial.test import sample
         self.test = sample.FooTest('test_foo')
         self.stream = StringIO.StringIO()
-        self.result = self.resultFactory(self.stream)
+        self.publisher = log.LogPublisher()
+        self.result = self.resultFactory(self.stream, publisher=self.publisher)
+
+
+    def test_shouldStopInitiallyFalse(self):
+        """
+        shouldStop is False to begin with.
+        """
+        self.assertEquals(False, self.result.shouldStop)
+
+
+    def test_shouldStopTrueAfterStop(self):
+        """
+        shouldStop becomes True soon as someone calls stop().
+        """
+        self.result.stop()
+        self.assertEquals(True, self.result.shouldStop)
+
+
+    def test_wasSuccessfulInitiallyTrue(self):
+        """
+        wasSuccessful() is True when there have been no results reported.
+        """
+        self.assertEquals(True, self.result.wasSuccessful())
+
+
+    def test_wasSuccessfulTrueAfterSuccesses(self):
+        """
+        wasSuccessful() is True when there have been only successes, False
+        otherwise.
+        """
+        self.result.addSuccess(self.test)
+        self.assertEquals(True, self.result.wasSuccessful())
+
+
+    def test_wasSuccessfulFalseAfterErrors(self):
+        """
+        wasSuccessful() becomes False after errors have been reported.
+        """
+        try:
+            1 / 0
+        except ZeroDivisionError:
+            self.result.addError(self.test, sys.exc_info())
+        self.assertEquals(False, self.result.wasSuccessful())
+
+
+    def test_wasSuccessfulFalseAfterFailures(self):
+        """
+        wasSuccessful() becomes False after failures have been reported.
+        """
+        try:
+            self.fail("foo")
+        except self.failureException:
+            self.result.addFailure(self.test, sys.exc_info())
+        self.assertEquals(False, self.result.wasSuccessful())
+
+
+
+class TestReporter(TestReporterInterface):
+    """
+    Tests for the base L{reporter.Reporter} class.
+    """
+
+    def setUp(self):
+        TestReporterInterface.setUp(self)
         self._timer = 0
         self.result._getTime = self._getTime
 
+
     def _getTime(self):
         self._timer += 1
         return self._timer
 
+
     def test_startStop(self):
         self.result.startTest(self.test)
         self.result.stopTest(self.test)
@@ -809,7 +925,6 @@ class TestReporter(unittest.TestCase):
         self.assertEqual(self.stream.getvalue(), 'Hello World!\n')
 
 
-
     def test_printErrorsDeprecated(self):
         """
         L{IReporter.printErrors} was deprecated in Twisted 8.0.
@@ -887,6 +1002,84 @@ class TestReporter(unittest.TestCase):
             __file__, f)
 
 
+    def test_warning(self):
+        """
+        L{reporter.Reporter} observes warnings emitted by the Twisted log
+        system and writes them to its output stream.
+        """
+        message = RuntimeWarning("some warning text")
+        category = 'exceptions.RuntimeWarning'
+        filename = "path/to/some/file.py"
+        lineno = 71
+        self.publisher.msg(
+            warning=message, category=category,
+            filename=filename, lineno=lineno)
+        self.assertEqual(
+            self.stream.getvalue(),
+            "%s:%d: %s: %s\n" % (
+                filename, lineno, category.split('.')[-1], message))
+
+
+    def test_duplicateWarningSuppressed(self):
+        """
+        A warning emitted twice within a single test is only written to the
+        stream once.
+        """
+        # Emit the warning and assert that it shows up
+        self.test_warning()
+        # Emit the warning again and assert that the stream still only has one
+        # warning on it.
+        self.test_warning()
+
+
+    def test_warningEmittedForNewTest(self):
+        """
+        A warning emitted again after a new test has started is written to the
+        stream again.
+        """
+        test = self.__class__('test_warningEmittedForNewTest')
+        self.result.startTest(test)
+
+        # Clear whatever startTest wrote to the stream
+        self.stream.seek(0)
+        self.stream.truncate()
+
+        # Emit a warning (and incidentally, assert that it was emitted)
+        self.test_warning()
+
+        # Clean up from the first warning to simplify the rest of the
+        # assertions.
+        self.stream.seek(0)
+        self.stream.truncate()
+
+        # Stop the first test and start another one (it just happens to be the
+        # same one, but that doesn't matter)
+        self.result.stopTest(test)
+        self.result.startTest(test)
+
+        # Clean up the stopTest/startTest output
+        self.stream.seek(0)
+        self.stream.truncate()
+
+        # Emit the warning again and make sure it shows up
+        self.test_warning()
+
+
+    def test_stopObserving(self):
+        """
+        L{reporter.Reporter} stops observing log events when its C{done} method
+        is called.
+        """
+        self.result.done()
+        self.stream.seek(0)
+        self.stream.truncate()
+        self.publisher.msg(
+            warning=RuntimeWarning("some message"),
+            category='exceptions.RuntimeWarning',
+            filename="file/name.py", lineno=17)
+        self.assertEqual(self.stream.getvalue(), "")
+
+
 
 class TestSafeStream(unittest.TestCase):
     def test_safe(self):
@@ -901,6 +1094,228 @@ class TestSafeStream(unittest.TestCase):
         self.assertEqual(stream.getvalue(), "Hello")
 
 
+
+class TestSubunitReporter(TestReporterInterface):
+    """
+    Tests for the subunit reporter.
+
+    This just tests that the subunit reporter implements the basic interface.
+    """
+
+    resultFactory = reporter.SubunitReporter
+
+
+    def setUp(self):
+        if reporter.TestProtocolClient is None:
+            raise SkipTest(
+                "Subunit not installed, cannot test SubunitReporter")
+        TestReporterInterface.setUp(self)
+
+
+    def assertForwardsToSubunit(self, methodName, *args, **kwargs):
+        """
+        Assert that 'methodName' on L{SubunitReporter} forwards to the
+        equivalent method on subunit.
+
+        Checks that the return value from subunit is returned from the
+        L{SubunitReporter} and that the reporter writes the same data to its
+        stream as subunit does to its own.
+
+        Assumes that the method on subunit has the same name as the method on
+        L{SubunitReporter}.
+        """
+        stream = StringIO.StringIO()
+        subunitClient = reporter.TestProtocolClient(stream)
+        subunitReturn = getattr(subunitClient, methodName)(*args, **kwargs)
+        subunitOutput = stream.getvalue()
+        reporterReturn = getattr(self.result, methodName)(*args, **kwargs)
+        self.assertEquals(subunitReturn, reporterReturn)
+        self.assertEquals(subunitOutput, self.stream.getvalue())
+
+
+    def test_subunitWithoutAddExpectedFailureInstalled(self):
+        """
+        Some versions of subunit don't have "addExpectedFailure". For these
+        versions, we report expected failures as successes.
+        """
+        try:
+            addExpectedFailure = reporter.TestProtocolClient.addExpectedFailure
+        except AttributeError:
+            # Then we've actually got one of those old versions installed, and
+            # the test is immediately applicable.
+            pass
+        else:
+            del reporter.TestProtocolClient.addExpectedFailure
+            self.addCleanup(
+                setattr, reporter.TestProtocolClient, 'addExpectedFailure',
+                addExpectedFailure)
+        try:
+            1 / 0
+        except ZeroDivisionError:
+            self.result.addExpectedFailure(self.test, sys.exc_info(), "todo")
+        expectedFailureOutput = self.stream.getvalue()
+        self.stream.truncate(0)
+        self.result.addSuccess(self.test)
+        successOutput = self.stream.getvalue()
+        self.assertEquals(successOutput, expectedFailureOutput)
+
+
+    def test_subunitWithoutAddSkipInstalled(self):
+        """
+        Some versions of subunit don't have "addSkip". For these versions, we
+        report skips as successes.
+        """
+        try:
+            addSkip = reporter.TestProtocolClient.addSkip
+        except AttributeError:
+            # Then we've actually got one of those old versions installed, and
+            # the test is immediately applicable.
+            pass
+        else:
+            del reporter.TestProtocolClient.addSkip
+            self.addCleanup(
+                setattr, reporter.TestProtocolClient, 'addSkip', addSkip)
+        self.result.addSkip(self.test, "reason")
+        skipOutput = self.stream.getvalue()
+        self.stream.truncate(0)
+        self.result.addSuccess(self.test)
+        successOutput = self.stream.getvalue()
+        self.assertEquals(successOutput, skipOutput)
+
+
+    def test_addExpectedFailurePassedThrough(self):
+        """
+        Some versions of subunit have "addExpectedFailure". For these
+        versions, when we call 'addExpectedFailure' on the test result, we
+        pass the error and test through to the subunit client.
+        """
+        addExpectedFailureCalls = []
+        def addExpectedFailure(test, error):
+            addExpectedFailureCalls.append((test, error))
+
+        # Provide our own addExpectedFailure, whether or not the locally
+        # installed subunit has addExpectedFailure.
+        self.result._subunit.addExpectedFailure = addExpectedFailure
+        try:
+            1 / 0
+        except ZeroDivisionError:
+            exc_info = sys.exc_info()
+            self.result.addExpectedFailure(self.test, exc_info, 'todo')
+        self.assertEquals(addExpectedFailureCalls, [(self.test, exc_info)])
+
+
+    def test_addSkipSendsSubunitAddSkip(self):
+        """
+        Some versions of subunit have "addSkip". For these versions, when we
+        call 'addSkip' on the test result, we pass the test and reason through
+        to the subunit client.
+        """
+        addSkipCalls = []
+        def addSkip(test, reason):
+            addSkipCalls.append((test, reason))
+
+        # Provide our own addSkip, whether or not the locally-installed
+        # subunit has addSkip.
+        self.result._subunit.addSkip = addSkip
+        self.result.addSkip(self.test, 'reason')
+        self.assertEquals(addSkipCalls, [(self.test, 'reason')])
+
+
+    def test_doneDoesNothing(self):
+        """
+        The subunit reporter doesn't need to print out a summary -- the stream
+        of results is everything. Thus, done() does nothing.
+        """
+        self.result.done()
+        self.assertEquals('', self.stream.getvalue())
+
+
+    def test_startTestSendsSubunitStartTest(self):
+        """
+        SubunitReporter.startTest() sends the subunit 'startTest' message.
+        """
+        self.assertForwardsToSubunit('startTest', self.test)
+
+
+    def test_stopTestSendsSubunitStopTest(self):
+        """
+        SubunitReporter.stopTest() sends the subunit 'stopTest' message.
+        """
+        self.assertForwardsToSubunit('stopTest', self.test)
+
+
+    def test_addSuccessSendsSubunitAddSuccess(self):
+        """
+        SubunitReporter.addSuccess() sends the subunit 'addSuccess' message.
+        """
+        self.assertForwardsToSubunit('addSuccess', self.test)
+
+
+    def test_addErrorSendsSubunitAddError(self):
+        """
+        SubunitReporter.addError() sends the subunit 'addError' message.
+        """
+        try:
+            1 / 0
+        except ZeroDivisionError:
+            error = sys.exc_info()
+        self.assertForwardsToSubunit('addError', self.test, error)
+
+
+    def test_addFailureSendsSubunitAddFailure(self):
+        """
+        SubunitReporter.addFailure() sends the subunit 'addFailure' message.
+        """
+        try:
+            self.fail('hello')
+        except self.failureException:
+            failure = sys.exc_info()
+        self.assertForwardsToSubunit('addFailure', self.test, failure)
+
+
+    def test_addUnexpectedSuccessSendsSubunitAddSuccess(self):
+        """
+        SubunitReporter.addFailure() sends the subunit 'addSuccess' message,
+        since subunit doesn't model unexpected success.
+        """
+        stream = StringIO.StringIO()
+        subunitClient = reporter.TestProtocolClient(stream)
+        subunitClient.addSuccess(self.test)
+        subunitOutput = stream.getvalue()
+        self.result.addUnexpectedSuccess(self.test, 'todo')
+        self.assertEquals(subunitOutput, self.stream.getvalue())
+
+
+    def test_loadTimeErrors(self):
+        """
+        Load-time errors are reported like normal errors.
+        """
+        test = runner.TestLoader().loadByName('doesntexist')
+        test.run(self.result)
+        output = self.stream.getvalue()
+        # Just check that 'doesntexist' is in the output, rather than
+        # assembling the expected stack trace.
+        self.assertIn('doesntexist', output)
+
+
+
+class TestSubunitReporterNotInstalled(unittest.TestCase):
+    """
+    Test behaviour when the subunit reporter is not installed.
+    """
+
+    def test_subunitNotInstalled(self):
+        """
+        If subunit is not installed, TestProtocolClient will be None, and
+        SubunitReporter will raise an error when you try to construct it.
+        """
+        stream = StringIO.StringIO()
+        self.patch(reporter, 'TestProtocolClient', None)
+        e = self.assertRaises(Exception, reporter.SubunitReporter, stream)
+        self.assertEquals("Subunit not available", str(e))
+
+
+
 class TestTimingReporter(TestReporter):
     resultFactory = reporter.TimingTextReporter
 
@@ -1029,3 +1444,130 @@ class TestAdaptedReporter(unittest.TestCase):
         """
         self.wrappedResult.addUnexpectedSuccess(self, Todo("no reason"))
         self.assertWrapped(self.wrappedResult, self)
+
+
+
+class FakeStream(object):
+    """
+    A fake stream which C{isatty} method returns some predictable.
+
+    @ivar tty: returned value of C{isatty}.
+    @type tty: C{bool}
+    """
+
+    def __init__(self, tty=True):
+        self.tty = tty
+
+
+    def isatty(self):
+        return self.tty
+
+
+
+class AnsiColorizerTests(unittest.TestCase):
+    """
+    Tests for L{reporter._AnsiColorizer}.
+    """
+
+    def setUp(self):
+        self.savedModules = sys.modules.copy()
+
+
+    def tearDown(self):
+        sys.modules.clear()
+        sys.modules.update(self.savedModules)
+
+
+    def test_supportedStdOutTTY(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{False} if the given
+        stream is not a TTY.
+        """
+        self.assertFalse(reporter._AnsiColorizer.supported(FakeStream(False)))
+
+
+    def test_supportedNoCurses(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{False} if the curses
+        module can't be imported.
+        """
+        sys.modules['curses'] = None
+        self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
+
+
+    def test_supportedSetupTerm(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{True} if
+        C{curses.tigetnum} returns more than 2 supported colors. It only tries
+        to call C{curses.setupterm} if C{curses.tigetnum} previously failed
+        with a C{curses.error}.
+        """
+        class fakecurses(object):
+            error = RuntimeError
+            setUp = 0
+
+            def setupterm(self):
+                self.setUp += 1
+
+            def tigetnum(self, value):
+                if self.setUp:
+                    return 3
+                else:
+                    raise self.error()
+
+        sys.modules['curses'] = fakecurses()
+        self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
+        self.assertTrue(reporter._AnsiColorizer.supported(FakeStream()))
+
+        self.assertEquals(sys.modules['curses'].setUp, 1)
+
+
+    def test_supportedTigetNumWrongError(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{False} and doesn't try
+        to call C{curses.setupterm} if C{curses.tigetnum} returns something
+        different than C{curses.error}.
+        """
+        class fakecurses(object):
+            error = RuntimeError
+
+            def tigetnum(self, value):
+                raise ValueError()
+
+        sys.modules['curses'] = fakecurses()
+        self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
+
+
+    def test_supportedTigetNumNotEnoughColor(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{False} if
+        C{curses.tigetnum} returns less than 2 supported colors.
+        """
+        class fakecurses(object):
+            error = RuntimeError
+
+            def tigetnum(self, value):
+                return 1
+
+        sys.modules['curses'] = fakecurses()
+        self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
+
+
+    def test_supportedTigetNumErrors(self):
+        """
+        L{reporter._AnsiColorizer.supported} returns C{False} if
+        C{curses.tigetnum} raises an error, and calls C{curses.setupterm} once.
+        """
+        class fakecurses(object):
+            error = RuntimeError
+            setUp = 0
+
+            def setupterm(self):
+                self.setUp += 1
+
+            def tigetnum(self, value):
+                raise self.error()
+
+        sys.modules['curses'] = fakecurses()
+        self.assertFalse(reporter._AnsiColorizer.supported(FakeStream()))
+        self.assertEquals(sys.modules['curses'].setUp, 1)
