@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -146,12 +146,8 @@ class TestSkipMethods(unittest.TestCase, ResultsTestMixin):
 class TestSkipClasses(unittest.TestCase, ResultsTestMixin):
     class SkippedClass(unittest.TestCase):
         skip = 'class'
-        def setUpClass(self):
-            self.__class__._setUpClassRan = True
         def setUp(self):
             self.__class__._setUpRan = True
-        def tearDownClass(self):
-            self.__class__._tearDownClassRan = True
         def test_skip1(self):
             raise unittest.SkipTest('skip1')
         def test_skip2(self):
@@ -162,90 +158,52 @@ class TestSkipClasses(unittest.TestCase, ResultsTestMixin):
         def test_skip4(self):
             raise RuntimeError("Skip me too")
 
+
     def setUp(self):
         self.loadSuite(TestSkipClasses.SkippedClass)
         TestSkipClasses.SkippedClass._setUpRan = False
-        TestSkipClasses.SkippedClass._setUpClassRan = False
-        TestSkipClasses.SkippedClass._tearDownClassRan = False
+
 
     def test_counting(self):
+        """
+        Skipped test methods still contribute to the total test count.
+        """
         self.assertCount(4)
 
+
     def test_setUpRan(self):
+        """
+        The C{setUp} method is not called if the class is set to skip.
+        """
         self.suite(self.reporter)
-        self.failUnlessEqual(TestSkipClasses.SkippedClass._setUpRan, False)
-        self.failUnlessEqual(TestSkipClasses.SkippedClass._setUpClassRan,
-                             False)
-        self.failUnlessEqual(TestSkipClasses.SkippedClass._tearDownClassRan,
-                             False)
+        self.assertFalse(TestSkipClasses.SkippedClass._setUpRan)
+
 
     def test_results(self):
+        """
+        Skipped test methods don't cause C{wasSuccessful} to return C{False},
+        nor do they contribute to the C{errors} or C{failures} of the reporter.
+        They do, however, add elements to the reporter's C{skips} list.
+        """
         self.suite(self.reporter)
         self.failUnless(self.reporter.wasSuccessful())
         self.failUnlessEqual(self.reporter.errors, [])
         self.failUnlessEqual(self.reporter.failures, [])
         self.failUnlessEqual(len(self.reporter.skips), 4)
 
+
     def test_reasons(self):
+        """
+        Test methods which raise L{unittest.SkipTest} or have their C{skip}
+        attribute set to something are skipped.
+        """
         self.suite(self.reporter)
         expectedReasons = ['class', 'skip2', 'class', 'class']
         # whitebox reporter
-        reasonsGiven = [ reason for test, reason in self.reporter.skips ]
-        self.failUnlessEqual(expectedReasons, reasonsGiven)
+        reasonsGiven = [reason for test, reason in self.reporter.skips]
+        self.assertEquals(expectedReasons, reasonsGiven)
 
 
-class TestSkipClassesRaised(unittest.TestCase, ResultsTestMixin):
-    class SkippedClass(unittest.TestCase):
-        def setUpClass(self):
-            raise unittest.SkipTest("class")
-        def setUp(self):
-            self.__class__._setUpRan = True
-        def tearDownClass(self):
-            self.__class__._tearDownClassRan = True
-        def test_skip1(self):
-            raise unittest.SkipTest('skip1')
-        def test_skip2(self):
-            raise RuntimeError("Ought to skip me")
-        test_skip2.skip = 'skip2'
-        def test_skip3(self):
-            pass
-        def test_skip4(self):
-            raise RuntimeError("Skip me too")
-
-    def setUp(self):
-        if hasattr(TestSkipClassesRaised.SkippedClass, 'skip'):
-            delattr(TestSkipClassesRaised.SkippedClass, 'skip')
-        self.loadSuite(TestSkipClassesRaised.SkippedClass)
-        TestSkipClassesRaised.SkippedClass._setUpRan = False
-        TestSkipClassesRaised.SkippedClass._tearDownClassRan = False
-
-    def test_counting(self):
-        self.assertCount(4)
-
-    def test_setUpRan(self):
-        self.suite(self.reporter)
-        self.failUnlessEqual(
-            TestSkipClassesRaised.SkippedClass._setUpRan, False)
-
-    def test_tearDownClassRan(self):
-        self.suite(self.reporter)
-        self.failUnlessEqual(
-            TestSkipClassesRaised.SkippedClass._tearDownClassRan, False)
-
-    def test_results(self):
-        self.suite(self.reporter)
-        self.failUnless(self.reporter.wasSuccessful())
-        self.failUnlessEqual(self.reporter.errors, [])
-        self.failUnlessEqual(self.reporter.failures, [])
-        self.failUnlessEqual(len(self.reporter.skips), 4)
-
-    def test_reasons(self):
-        self.suite(self.reporter)
-        expectedReasons = ['class', 'skip2', 'class', 'class']
-        # whitebox reporter
-        reasonsGiven = [ reason for test, reason in self.reporter.skips ]
-        self.failUnlessEqual(expectedReasons, reasonsGiven)
-
 
 class TestTodo(unittest.TestCase, ResultsTestMixin):
     class TodoTests(unittest.TestCase):
@@ -465,39 +423,6 @@ class FixtureTest(unittest.TestCase):
         self.assert_(isinstance(errors[0][1].value, erroneous.FoolishError))
 
 
-    def testBrokenSetUpClass(self):
-        """
-        When setUpClass fails, an error is recorded in the result object.
-        """
-        suite = self.loader.loadClass(erroneous.TestFailureInSetUpClass)
-        suite.run(self.reporter)
-        self.assert_(self.reporter.errors)
-
-
-    def testBrokenTearDownClass(self):
-        """
-        When setUpClass fails, an error is recorded in the result object.
-        """
-        suite = self.loader.loadClass(erroneous.TestFailureInTearDownClass)
-        suite.run(self.reporter)
-        self.assert_(self.reporter.errors)
-
-
-
-class FixtureMetaTest(unittest.TestCase):
-
-    def test_testBrokenTearDownClass(self):
-        """FixtureTest.testBrokenTearDownClass succeeds when run twice
-        """
-        test = FixtureTest('testBrokenTearDownClass')
-        result = reporter.TestResult()
-        test(result)
-        self.failUnless(result.wasSuccessful())
-        result2 = reporter.TestResult()
-        test(result2)
-        self.failUnless(result2.wasSuccessful())
-
-
 
 class SuppressionTest(unittest.TestCase):
 
@@ -506,50 +431,77 @@ class SuppressionTest(unittest.TestCase):
 
 
     def setUp(self):
-        self.stream = StringIO.StringIO()
-        self._stdout, sys.stdout = sys.stdout, self.stream
         self.loader = runner.TestLoader()
 
 
-    def tearDown(self):
-        sys.stdout = self._stdout
-        self.stream = None
-
-
-    def getIO(self):
-        return self.stream.getvalue()
-
-
-    def testSuppressMethod(self):
+    def test_suppressMethod(self):
+        """
+        A suppression set on a test method prevents warnings emitted by that
+        test method which the suppression matches from being emitted.
+        """
         self.runTests(self.loader.loadMethod(
             suppression.TestSuppression.testSuppressMethod))
-        self.assertNotSubstring(suppression.METHOD_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.CLASS_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.MODULE_WARNING_MSG, self.getIO())
+        warningsShown = self.flushWarnings([
+                suppression.TestSuppression._emit])
+        self.assertEqual(
+            warningsShown[0]['message'], suppression.CLASS_WARNING_MSG)
+        self.assertEqual(
+            warningsShown[1]['message'], suppression.MODULE_WARNING_MSG)
+        self.assertEqual(len(warningsShown), 2)
 
 
-    def testSuppressClass(self):
+    def test_suppressClass(self):
+        """
+        A suppression set on a L{TestCase} subclass prevents warnings emitted
+        by any test methods defined on that class which match the suppression
+        from being emitted.
+        """
         self.runTests(self.loader.loadMethod(
             suppression.TestSuppression.testSuppressClass))
-        self.assertSubstring(suppression.METHOD_WARNING_MSG, self.getIO())
-        self.assertNotSubstring(suppression.CLASS_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.MODULE_WARNING_MSG, self.getIO())
+        warningsShown = self.flushWarnings([
+                suppression.TestSuppression._emit])
+        self.assertEqual(
+            warningsShown[0]['message'], suppression.METHOD_WARNING_MSG)
+        self.assertEqual(
+            warningsShown[1]['message'], suppression.MODULE_WARNING_MSG)
+        self.assertEqual(len(warningsShown), 2)
 
 
-    def testSuppressModule(self):
+    def test_suppressModule(self):
+        """
+        A suppression set on a module prevents warnings emitted by any test
+        mewthods defined in that module which match the suppression from being
+        emitted.
+        """
         self.runTests(self.loader.loadMethod(
             suppression.TestSuppression2.testSuppressModule))
-        self.assertSubstring(suppression.METHOD_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.CLASS_WARNING_MSG, self.getIO())
-        self.assertNotSubstring(suppression.MODULE_WARNING_MSG, self.getIO())
+        warningsShown = self.flushWarnings([
+                suppression.TestSuppression._emit])
+        self.assertEqual(
+            warningsShown[0]['message'], suppression.METHOD_WARNING_MSG)
+        self.assertEqual(
+            warningsShown[1]['message'], suppression.CLASS_WARNING_MSG)
+        self.assertEqual(len(warningsShown), 2)
 
 
-    def testOverrideSuppressClass(self):
-        self.runTests(self.loader.loadMethod(
-            suppression.TestSuppression.testOverrideSuppressClass))
-        self.assertSubstring(suppression.CLASS_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.MODULE_WARNING_MSG, self.getIO())
-        self.assertSubstring(suppression.METHOD_WARNING_MSG, self.getIO())
+    def test_overrideSuppressClass(self):
+        """
+        The suppression set on a test method completely overrides a suppression
+        with wider scope; if it does not match a warning emitted by that test
+        method, the warning is emitted, even if a wider suppression matches.
+        """
+        case = self.loader.loadMethod(
+            suppression.TestSuppression.testOverrideSuppressClass)
+        self.runTests(case)
+        warningsShown = self.flushWarnings([
+                suppression.TestSuppression._emit])
+        self.assertEqual(
+            warningsShown[0]['message'], suppression.METHOD_WARNING_MSG)
+        self.assertEqual(
+            warningsShown[1]['message'], suppression.CLASS_WARNING_MSG)
+        self.assertEqual(
+            warningsShown[2]['message'], suppression.MODULE_WARNING_MSG)
+        self.assertEqual(len(warningsShown), 3)
 
 
 
@@ -572,8 +524,6 @@ class GCMixin:
             self._log('test1')
         def test_2(self):
             self._log('test2')
-        def tearDownClass(self):
-            self._log('tearDownClass')
 
     def _log(self, msg):
         self._collectCalled.append(msg)
@@ -620,24 +570,6 @@ class TestGarbageCollection(GCMixin, unittest.TestCase):
             self._collectCalled,
             ['collect', 'setUp', 'test', 'tearDown', 'collect'])
 
-    def test_collectCalledWhenTearDownClass(self):
-        """
-        test gc.collect is called after tearDownClass.
-        """
-        test = unittest.TestSuite(
-            [TestGarbageCollection.ClassTest('test_1'),
-             TestGarbageCollection.ClassTest('test_2')])
-        test = unittest.decorate(
-            test, unittest._ForceGarbageCollectionDecorator)
-        result = reporter.TestResult()
-        test.run(result)
-        # check that collect gets called after individual tests, and
-        # after tearDownClass
-        self.failUnlessEqual(
-            self._collectCalled,
-            ['collect', 'test1', 'collect',
-             'collect', 'test2', 'tearDownClass', 'collect'])
-
 
 
 class TestUnhandledDeferred(unittest.TestCase):
@@ -817,6 +749,43 @@ class TestAddCleanup(unittest.TestCase):
 
 
 
+class TestSuiteClearing(unittest.TestCase):
+    """
+    Tests for our extension that allows us to clear out a L{TestSuite}.
+    """
+
+
+    def test_clearSuite(self):
+        """
+        Calling L{unittest._clearSuite} on a populated L{TestSuite} removes
+        all tests.
+        """
+        suite = unittest.TestSuite()
+        suite.addTest(unittest.TestCase())
+        # Double check that the test suite actually has something in it.
+        self.assertEqual(1, suite.countTestCases())
+        unittest._clearSuite(suite)
+        self.assertEqual(0, suite.countTestCases())
+
+
+    def test_clearPyunitSuite(self):
+        """
+        Calling L{unittest._clearSuite} on a populated standard library
+        L{TestSuite} removes all tests.
+
+        This test is important since C{_clearSuite} operates by mutating
+        internal variables.
+        """
+        pyunit = __import__('unittest')
+        suite = pyunit.TestSuite()
+        suite.addTest(unittest.TestCase())
+        # Double check that the test suite actually has something in it.
+        self.assertEqual(1, suite.countTestCases())
+        unittest._clearSuite(suite)
+        self.assertEqual(0, suite.countTestCases())
+
+
+
 class TestTestDecorator(unittest.TestCase):
     """
     Tests for our test decoration features.
@@ -829,7 +798,11 @@ class TestTestDecorator(unittest.TestCase):
         """
         self.assertEqual(observed.__class__, expected.__class__,
                          "Different class")
-        self.assertIdentical(observed._originalTest, expected._originalTest)
+        observedOriginal = getattr(observed, '_originalTest', None)
+        expectedOriginal = getattr(expected, '_originalTest', None)
+        self.assertIdentical(observedOriginal, expectedOriginal)
+        if observedOriginal is expectedOriginal is None:
+            self.assertIdentical(observed, expected)
 
 
     def assertSuitesEqual(self, observed, expected):
@@ -899,6 +872,41 @@ class TestTestDecorator(unittest.TestCase):
             decoratedTest, unittest.TestSuite([unittest.TestDecorator(test)]))
 
 
+    def test_decorateInPlaceMutatesOriginal(self):
+        """
+        Calling L{decorate} on a test suite will mutate the original suite.
+        """
+        test = unittest.TestCase()
+        suite = unittest.TestSuite([test])
+        decoratedTest = unittest.decorate(
+            suite, unittest.TestDecorator)
+        self.assertSuitesEqual(
+            decoratedTest, unittest.TestSuite([unittest.TestDecorator(test)]))
+        self.assertSuitesEqual(
+            suite, unittest.TestSuite([unittest.TestDecorator(test)]))
+
+
+    def test_decorateTestSuiteReferences(self):
+        """
+        When decorating a test suite in-place, the number of references to the
+        test objects in that test suite should stay the same.
+
+        Previously, L{unittest.decorate} recreated a test suite, so the
+        original suite kept references to the test objects. This test is here
+        to ensure the problem doesn't reappear again.
+        """
+        getrefcount = getattr(sys, 'getrefcount', None)
+        if getrefcount is None:
+            raise unittest.SkipTest(
+                "getrefcount not supported on this platform")
+        test = unittest.TestCase()
+        suite = unittest.TestSuite([test])
+        count1 = getrefcount(test)
+        decoratedTest = unittest.decorate(suite, unittest.TestDecorator)
+        count2 = getrefcount(test)
+        self.assertEquals(count1, count2)
+
+
     def test_decorateNestedTestSuite(self):
         """
         Calling L{decorate} on a test suite with nested suites will return a
